#!/bin/bash
# mktemplate.sh - make reduction template file for MCSMDP
# 2007. 5. 11 YOSHIKAWA, Tomohiro
# $Id: mktemplate.sh,v 1.5 2007-09-03 08:52:48 tomohiro Exp $
function usage(){
cat <<EOF
usage:
        ${0##/*/} [config file] > [template file]
EOF
}

if [ $# -lt 1 ]; then
    usage
    exit
fi

source $1

mdpname=$(basename $mdpfile .mdp)

cat <<EOF
iraf.mcsmdp()

hselect *.fits \$I "@'DET-ID' = 1 & K_DITCNT = 1" > files1a.list
hselect *.fits \$I "@'DET-ID' = 1 & K_DITCNT = 2" > files1b.list
hselect *.fits \$I "@'DET-ID' = 2 & K_DITCNT = 1" > files2a.list
hselect *.fits \$I "@'DET-ID' = 2 & K_DITCNT = 2" > files2b.list

## pre-reduction
unlearn mdpproc
iraf.mdpproc.bpmask="$bpm1"
iraf.mdpproc.gdb="$gdb1"
iraf.mdpproc.gmp="$gmp1"
iraf.mdpproc.flat="$flat1"
mdpproc @files1a.list @files1b.list

unlearn mdpproc
iraf.mdpproc.bpmask="$bpm2"
iraf.mdpproc.gdb="$gdb2"
iraf.mdpproc.gmp="$gmp2"
iraf.mdpproc.flat="$flat2"
mdpproc @files2a.list @files2b.list

mklist gcflabMCSA????????.fits expr="@'DET-ID' = 1" > gcflab1.list
mklist gcflabMCSA????????.fits expr="@'DET-ID' = 2" > gcflab2.list

#########
## ch1 ##
#########

## transform mdpfile
mdptran $mdpfile ${mdpname}tr1.mdp @gcflab1.list

## cutout & combine spectrums
unlearn mdpcombine
iraf.mdpcombine.coordlist="$coordlist"
iraf.mdpcombine.ymin=42
iraf.mdpcombine.ymax=2003

# put [object name] for manual OH emission line identification
mdpcombine @gcflab1.list ${mdpname}tr1.mdp ${prefix}_[object name] [object name] id_mode=manual

iraf.mdpcombine.id_mode='auto'
iraf.mdpcombine.refname='[object name]'
iraf.mdpcombine.bg_inter=no

## execute and paste the outputs to the log file
!awk '{print \$8}' ${mdpname}tr1.mdp | sed 's/\(.*\)/mdpcombine\ \@gcflab1\.list\ '${mdpname}'tr1\.mdp\ '${prefix}'_\1 \1/'


### check rotation of grism (optional)
#!sed  's/\(.*\)/r\1/' gcflab1.list > rgcflab1.list
#mdprotate @gcflab1.list  @rgcflab1.list __enter_rotation_here__ interpolant=linear boundary=constant
#
#mdptran $mdpfile ${mdpname}tr1.mdp @rgcflab1.list
#
### combine frames
#unlearn mdpcombine
#iraf.mdpcombine.coordlist="$coordlist"
#iraf.mdpcombine.ymin=42
#iraf.mdpcombine.ymax=2003
#
#!awk '{print \$8}' ${mdpname}tr1.mdp | sed 's/\(.*\)/mdpcombine\ \@rgcflab1\.list\ '${mdpname}'tr1\.mdp\ '${prefix}'_\1 \1/'

#########
## ch2 ##
#########

## transform mdpfile
mdptran $mdpfile ${mdpname}tr2.mdp @gcflab2.list

## cutout & combine spectrums
unlearn mdpcombine
iraf.mdpcombine.coordlist="$coordlist"
iraf.mdpcombine.ymin=28
iraf.mdpcombine.ymax=2008

# put [object name] for manual OH emission line identification
mdpcombine @gcflab2.list ${mdpname}tr2.mdp ${prefix}_[object name] [object name] id_mode=manual

iraf.mdpcombine.id_mode='auto'
iraf.mdpcombine.refname='[object name]'
iraf.mdpcombine.bg_inter=no

## execute and paste the outputs to the log file
!awk '{print \$8}' ${mdpname}tr2.mdp | sed 's/\(.*\)/mdpcombine\ \@gcflab2\.list\ '${mdpname}'tr2\.mdp\ '${prefix}'_\1 \1/'

### check rotation of grism (optional)
#!sed  's/\(.*\)/r\1/' gcflab2.list > rgcflab2.list
#mdprotate @gcflab2.list  @rgcflab2.list __enter_rotation_here__ interpolant=linear boundary=constant
#
#mdptran $mdpfile ${mdpname}tr2.mdp @rgcflab2.list
#
### combine frames
#unlearn mdpcombine
#iraf.mdpcombine.coordlist="$coordlist"
#iraf.mdpcombine.ymin=28
#iraf.mdpcombine.ymax=2008
#
#!awk '{print \$8}' ${mdpname}tr2.mdp | sed 's/\(.*\)/mdpcombine\ \@gcflab2\.list\ '${mdpname}'tr2\.mdp\ '${prefix}'_\1 \1/'
EOF
