/*********************************************************
 *  MOIRCS Acquisition:
 *
 *   Apply Gaussian filter to the image data array
 *
 *       Input  : *fPix_val   :
 *       Input  : *iAxlen     : Image XY axis length
 *       Input  : fSigma      :
 *       Output : *fpix_val_g : 
 *
 *       Ver.0.0 : 20050123 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

int ac_gaussfilter(const float *fPix_val, const int *iAxlen,
   const float fSigma, float *fPix_val_g )
{
    int       i, j, k, l;
    float     fScale = 1.0;
    int       iWindow;
    float     fCenter;
    int       iKnum, iLnum;
    float     fDist;
    float     *fGauss; // Weight determined with normalized Gaussian distribution

#ifdef DEBUG
    fprintf(stderr, " ac_gaussfilter : Initialize array \n");
#endif
   for(i=1; i<=*(iAxlen+0); i++ ){
      for(j=1; j<=*(iAxlen+1); j++ ){
         *(fPix_val_g + (j-1)* *(iAxlen+0) + (i-1)) = 0.0;
      }
   }

#ifdef DEBUG
    fprintf(stderr, " ac_gaussfilter : Make Gaussian filter array \n");
#endif 
   iWindow = 2 * floor( fScale * fSigma ) + 1; // Gaussian filter box size, must be odd
   fCenter = iWindow/2.0+0.5; // Center of the window

#ifdef DEBUG
    fprintf(stderr, " ac_gaussfilter : Size of the filter = %d \n", iWindow);
#endif 

   fGauss = (float *)malloc(sizeof(float)*iWindow*iWindow);
   for(i=1; i<=iWindow; i++){
      for(j=1; j<=iWindow; j++){
         fDist = fScale * sqrt((fCenter-i)*(fCenter-i)+(fCenter-j)*(fCenter-j));
         *(fGauss + (j-1)*iWindow + (i-1)) = 
              1/sqrt(2.0*3.14*fSigma) * exp( fDist*fDist/2.0/fSigma/fSigma );
      }
   }
#ifdef DEBUG
    fprintf(stderr, " ac_gaussfilter : Apply Gaussian filter \n");
#endif
    for(i=1; i<=*(iAxlen+0); i++ ){
       for(j=1; j<=*(iAxlen+1); j++ ){

          iKnum = 1;
  
          for(k=(i-floor(iWindow/2.0)); k<=(i+floor(iWindow/2.0)); k++ ){
             if( 1<=k && k<=*(iAxlen+0) ){

                iLnum = 1;
                for(l=(j-floor(iWindow/2.0)); l<=(j+floor(iWindow/2.0)); l++ ){
                   if( 1<=l && l<=*(iAxlen+1) ){

                      *(fPix_val_g + (l-1)* *(iAxlen+0) + (k-1)) = 
                         + *(fPix_val_g + (l-1)* *(iAxlen+0) + (k-1)) 
                         + *(fPix_val + (j-1)* *(iAxlen+0) + (i-1)) 
                          * *(fGauss + (iLnum-1)*iWindow + (iKnum-1));
                   }
                   iLnum = iLnum + 1;
                } 
             }
             iKnum = iKnum + 1;
          }

       }
    }
           
    free( fGauss );
    return EXIT_SUCCESS;
}
