/*********************************************************
 *  MOIRCS Acquisition:
 *
 *    Pick up 1st hole manually
 *     calculate new hole positions
 *
 *       Input  : *pix_val  :
 *       Input  : *axlen    : Image XY axis length
 *       Input  : cLine     :
 *       Input  : iNhole    : Number of holes
 *       Input  : pHoleX    : Hole X coordinate
 *       Input  : pHoleY    : Hole Y coordinate
 *       Output :           :
 *
 *
 *       Ver.0.0 : 20050123 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

int ac_pickhole(float *pix_val,
   const int *axlen, const char *cLine,
   const int iNhole, float *pHoleX, float *pHoleY)
{
    int       iN;
    float     fmin, fmax;
    static    float tr[6] = {0.0, 1.0, 0.0, 0.0, 0.0, 1.0};
    float     fXclick, fYclick;
    float     fXshift, fYshift;
    float     fNewX, fNewY;
    float     fXmin_vp, fXmax_vp, fYmin_vp, fYmax_vp;
    int       iWidth;
    int       iXmin, iXmax, iYmin, iYmax;
    char      end;
    char      cText[100];
    float     sigma_rej;
    int       ndata;
    float     average,sigma;

    int       iNxview;
    float     fXshift_vp, fYshift_vp;
    float     fRotate;

    float     fRhole;

    int       iCI;

#ifdef DEBUG_AC_PICKHOLE
    printf(" ac_pickhole : determin display range in %d x %d image \n", 
          *(axlen+0), *(axlen+1));
#endif

    ndata = *(axlen+0) * *(axlen+1);
    sigma_rej = 0.0;

    ac_sigclip(pix_val, ndata, sigma_rej, 0, &average, &sigma);
    fmin = average - 2 * sigma;
    fmax = average + 4 * sigma;

#ifdef DEBUG_AC_PICKHOLE
    printf(" ac_pickhole : image average = %f sigma = %f \n ", average,sigma);
    printf(" ac_pickhole : in the displayed region with  \n");
    printf(" ac_pickhole : %f sigma rejection.  \n", sigma_rej);
#endif

    end = 'n';
   
    fRotate = 0.0;

#ifdef DEBUG_AC_PICKHOLE
    fprintf(stderr, " ac_pickhole : Show Whole Image \n");
#endif
    cpgpage();
    fXmin_vp = 0.1;
    fXmax_vp = 0.1 + 0.7;
    fYmin_vp = 0.1;
    fYmax_vp = 0.1 + 1.2 * 0.7 * *(axlen+1) / *(axlen+0);
    if( fYmax_vp > 1.0 ){
        fXmin_vp = 0.1;
        fXmax_vp = 0.1 + 0.7 * *(axlen+0) / *(axlen+1);
        fYmin_vp = 0.1;
        fYmax_vp = 0.1 + 1.2 * 0.7;
    }
    cpgsvp(fXmin_vp, fXmax_vp, fYmin_vp, fYmax_vp);
    cpgswin( 1, *(axlen+0), 1, *(axlen+1) );
    cpggray(pix_val, *(axlen+0), *(axlen+1), 1, *(axlen+0), 1, *(axlen+1), fmin , fmax, tr);

    cpgswin( 0, 1, 0, 1);
    cpgsch(0.8);
    sprintf(cText, "STEP 1 : AC_PICKOBJ ");
    cpgptxt( 1.05, 0.97, 0.0, 0.0, cText );
    sprintf(cText, "       : MEASURE THE POSITION OF HOLE 1 ");
    cpgptxt( 1.05, 0.94, 0.0, 0.0, cText );
    cpgsch(0.6);
    sprintf(cText, "Left click at center of hole 1 in the left image");
    cpgptxt( 1.05, 0.88, 0.0, 0.0, cText );
    sprintf(cText, "3 or 4 to Rotate : Current rotation = %f deg", fRotate);
    cpgptxt( 1.05, 0.85, 0.0, 0.0, cText );
    sprintf(cText, "Right click to step 2");
    cpgptxt( 1.05, 0.82, 0.0, 0.0, cText );
    sprintf(cText, "!! Bright area is displayed in black !!");
    cpgptxt( 1.05, 0.79, 0.0, 0.0, cText );

    end = 'i';

    iCI = 1;

    while(end != 88){
       if( end == 65 || end == '3' || end == '4' || end == 'i' ){
          cpgsch(0.7);
          cpgsci(iCI);
          cpgpt( iNhole, pHoleX, pHoleY, 26 );
          for( iN=0; iN<iNhole; iN++ ){
             sprintf(cText, "%d", iN+1);
             cpgptxt( *(pHoleX+iN), *(pHoleY+iN), 0.0, -1.0, cText );
          }
          iCI = iCI + 1;
          cpgsci(1);

#ifdef DEBUG_AC_PICKHOLE
    fprintf(stderr, " ac_pickhole : Draw sub-area \n");
#endif
          iNxview = 0;
          fXshift_vp = 0.0;
          fYshift_vp = 0.0;
          for(iN=0; iN<iNhole; iN++){
             cpgsvp(fXmax_vp+0.05+fXshift_vp, fXmax_vp+0.20+fXshift_vp, 0.5-fYshift_vp, 0.5+0.15*1.3-fYshift_vp); 
             fRhole = DEF_RHOLE_MOIRCS;
             iWidth = SIZE_SUBARRAY*fRhole;

             iXmin = ceil(*(pHoleX+iN) - iWidth/2.0);
             iXmax = iXmin + iWidth-1;
             iYmin = ceil(*(pHoleY+iN) - iWidth/2.0);
             iYmax = iYmin + iWidth-1;
             cpgswin( iXmin, iXmax, iYmin, iYmax );
             cpggray(pix_val, *(axlen+0), *(axlen+1), 1, *(axlen+0), 1, *(axlen+1), fmin , fmax, tr);
	     cpgsci(4);
             cpgsfs(2);
             cpgcirc( *(pHoleX+iN), *(pHoleY+iN), fRhole );
  
             fXshift_vp = fXshift_vp + 0.15;
             iNxview = iNxview + 1;
             if( iNxview == 3 ){
                fXshift_vp = 0;
                fYshift_vp = fYshift_vp + 0.15*1.3 + 0.02;
                iNxview = 0;
             }
          }
       }
        
       cpgsvp(fXmin_vp, fXmax_vp, fYmin_vp, fYmax_vp);
       cpgswin( 1, *(axlen+0), 1, *(axlen+1) );
       cpgcurs(&fXclick, &fYclick, &end);

#ifdef DEBUG_AC_PICKHOLE
    fprintf(stderr, " ac_pickhole : Key %d,  Clicked position %f %f \n", end, fXclick, fYclick );
#endif

       if( end == '3' ){
          for(iN=0;iN<iNhole;iN++){
             fNewX = *(pHoleX+iN)*cos(0.1*3.14/180.0)+*(pHoleY+iN)*sin(0.1*3.14/180.0);
             fNewY = -*(pHoleY+iN)*sin(0.1*3.14/180.0)+*(pHoleY+iN)*cos(0.1*3.14/180.0);
             *(pHoleX+iN) = fNewX;
             *(pHoleY+iN) = fNewY;
          }
          fRotate = fRotate + 0.1;
       }
       else if(end == '4'){
          for(iN=0;iN<iNhole;iN++){
             fNewX = *(pHoleX+iN)*cos(-0.1*3.14/180.0)+*(pHoleY+iN)*sin(-0.1*3.14/180.0);
             fNewY = -*(pHoleY+iN)*sin(-0.1*3.14/180.0)+*(pHoleY+iN)*cos(-0.1*3.14/180.0);
             *(pHoleX+iN) = fNewX;
             *(pHoleY+iN) = fNewY;
          }
          fRotate = fRotate - 0.1;
          
       }
       else if(end == 65){
          fXshift = fXclick - *(pHoleX+0);
          fYshift = fYclick - *(pHoleY+0);
          for(iN=0;iN<iNhole;iN++){
              *(pHoleX+iN) = *(pHoleX+iN)+fXshift;
              *(pHoleY+iN) = *(pHoleY+iN)+fYshift;
          }
       }
    }
    return EXIT_SUCCESS;
}
