/*********************************************************
  * Aperture Photometory:
 *
 *    Sigma Clipping
 *
 *       Input  : *pData    : Data
 *       Input  : iNdata    : Number of Data
 *       Input  : fRejsig   : Rejection Sigma
 *       Input  : dMode     : AVERAGE:0 MODE:1
 *       Output : *fMean    : Mean Value
 *       Output : *fSigma   : RMS
 *
 *       Ver.2.0 : 20040303 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

void ac_sigclip(const float *pData, const int iNdata,
                const float fRejsig, const int dMode,
                  float *fMean, float *fSigma)
{
    int     i;
    int     *pFlag;
    float   fValue;
    float   fDev_sum, fRMS;
    float   *pData_clip;
    int     iNclip, iNremain, iNcycle;

/* Initialization */
    pFlag = (int *)malloc(sizeof(int)*(iNdata));
    pData_clip = (float *)malloc(sizeof(float)*(iNdata));

    for(i=0; i<=iNdata-1; i++){
        *(pFlag+i) = 0;
    }

    iNcycle = 0;
    iNclip = 1;

    while( iNclip > 0 && iNcycle <= 50 ){
       iNremain = 0;
       fValue = 0;
       fDev_sum = 0;
       fRMS = 0;
       iNclip = 0;

/*
 * Clip Data and Determin Average or Mode
 */
       for(i=0; i<iNdata; i++){
           if( *(pFlag+i) == 0 ){
               *(pData_clip + iNremain) = *(pData+i);
               iNremain = iNremain + 1;
           }
       }
       if( dMode == 0 ){
            fValue = ac_average( pData_clip, iNremain );
       }
       else if( dMode == 1){
            fValue = ac_mode( pData_clip, iNremain );
       }

/*
 * Determin RMS for Clipped Data
 */
       for(i=0; i<iNremain; i++){
           fDev_sum = fDev_sum + (*(pData_clip+i) - fValue)*(*(pData_clip+i) - fValue);
       }
       fRMS = sqrt( fDev_sum / iNremain );

#ifdef DEBUG_SIGCLIP
       fprintf(stderr," ac_sigclip : iNcycle=%d fValue=%f RMS=%f \n", iNcycle, fValue, fRMS);
#endif

/*
 * Clipping Data
 */
       for(i=0; i<=iNdata; i++){
           if ( *(pData+i) >= fValue + fRejsig * fRMS && *(pFlag+i) == 0){
                *(pFlag+i) = 1;
                iNclip = iNclip + 1; 
           }
           if ( *(pData+i) <= fValue - fRejsig * fRMS && *(pFlag+i) == 0){
                *(pFlag+i) = 1;
                iNclip = iNclip + 1;
           }
       }
       iNcycle = iNcycle+1;
  
       if(fRejsig<=0.1){
           iNcycle = 100.0;
       }
       if(fabs(fValue)<1.0e-5 && fabs(fRMS)<1.0e-5){
           iNcycle = 100.0;
       }
     }

     *fMean = fValue;
     *fSigma = fRMS;

     free(pFlag);
     free(pData_clip);
     return;
}    
