/*********************************************************
 * MOIRCS acquisition:
 *
 *     Determine Threshold using pixel value
 *
 *       Input  : *pix_val  :
 *       Input  : iNx       : x-dimension of pix_val
 *       Input  : iNy       : y-dimention of pix_val
 *       Input  : fRhole     : 
 *       Input  : fCenOld_x :
 *       Input  : fCenOld_y :  
 *       Output : fThresh   : Threshold for determining center
 *
 *
 *       Ver.0.0 : 20050123 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

void ac_thresh2(const float *pPix_org, const int *pMask,
             const int iNx, const int iNy,
             const float fRhole,
             const float fCen_x, const float fCen_y,
             float *fThresh)
{  

   int     i,j,k;
   float   fDist;
   float   fScale;

   int     iNpix;
   float   *pPix; 
   float   fSig_rej, fRMS, fAverage;

#ifdef DEBUG_AC_THRESH2
   printf("ac_thresh2 : Determine fThresh r=%f\n", fRhole);
#endif

/*
 * Make New Array of Pixel Data outside of fRhole region
 */ 
   pPix = (float *)malloc(sizeof(float)*(iNx * iNy));
   for (i=0; i<= iNx * iNy -1; i++){
      *(pPix+i) = 0.0;
   }

   iNpix = 0;
   fScale = 1.0;
   for (j=1; j<=iNy; j++){
       for (i=1; i<=iNx; i++){

           fDist = sqrt((i*fScale - fCen_x + 0.5)*(i*fScale - fCen_x + 0.5)
                        +(j*fScale - fCen_y + 0.5)*(j*fScale - fCen_y + 0.5));
 
           if ( fDist < fRhole ){
               k = (j-1)*iNx + (i-1); /* Fortran convention */
               if ( *(pMask+k)!=MASK_VALUE ){
                  *(pPix+iNpix) = *(pPix_org+k);
                  iNpix = iNpix+1;
               }
           }
       }
   }
   
   fSig_rej = 3.0;

   ac_sigclip( pPix, iNpix, fSig_rej, 0, &fAverage, &fRMS );

#ifdef DEBUG_AC_THRESH2
   fprintf(stderr,"ac_thresh2 : fRMS = %f  \n", fRMS);
#endif
   *fThresh = 1.0* fRMS + fAverage;

   free(pPix);
   return;
}
