/*****************************************************
 *  MOIRCS Acquisition:
 *
 *   Read Data File 
 *
 *   Input : cFile   : Input Data File Name
 *   Output: fHoleX  : X coordinate list
 *   Output: fHoleY  : Y coordinate list
 *   Output: iStat   : <0 if iC=0, or if fileread err
 *                   : =iC : Number of data read
 *
 *     Version 1.0 20050915 Masayuki Akiyama
 *****************************************************
 */

#include  "acquisition.h"

#undef DEBUG

int file_read_sbr(const char *cFile, float *fHoleX, float *fHoleY, const char *cType)
{
   int    iNum;
   FILE   *pFile;
   char   cLine[1000];
   char   cData[30][10];
   int    iStart;       /* Reading status */
   int    iStat;
   int    iC;
   float  fRawX, fRawY;
   float  fX, fY;

   int    iChar, iNchar, iData;

   iStart = 0;
   iStat = 0;
   iC = 0;

   pFile = fopen(cFile, "r");
   
   while(fgets (cLine, 1000, pFile)){
      iNum = sscanf(cLine, "%s ", cData[0]);
      if( iNum < 1){
          iStat = -2;
          break;
      }
      else{
         iStart = 1;
         if( strstr( cLine, cType ) != NULL){
//            iNum = sscanf(cLine, "%s\,%s\,%s\,", cData[0], cData[1], cData[2]);
            iData = 0;
            iNchar = 0;
            iChar = 0;
            while( cLine[iChar] != '\n' ){
                if( cLine[iChar] == ',' ){
                    cData[iData][iNchar] = '\0';
                    iNchar = 0;
                    iData = iData + 1;
                }
                else{
                    cData[iData][iNchar] = cLine[iChar];
                    iNchar = iNchar + 1;
                }
                iChar = iChar +1;
            }
// Convert SBR coordinates to MOIRCS MDP coordinates
#ifdef DEBUG_FILE_READ_SBR
         fprintf(stdout, "%s %s %s \n", cData[0], cData[1], cData[2]);
#endif
          fX = 1078.0 - atof(cData[1])*17.57789;
          fY = 1857.0 + atof(cData[2])*17.57789;
          *(fHoleX+iC) = 365.0+ (fX-300.0)*cos(SBRROT/180.0*3.1415) - (fY-2660.0)*sin(SBRROT/180.0*3.1415);
          *(fHoleY+iC) = 2580.0 + (fX-300.0)*sin(SBRROT/180.0*3.1415) + (fY-2660.0)*cos(SBRROT/180.0*3.1415);
//        *(fHoleX+iC) = 1078.0 - atof(cData[1])*17.57789;
//        *(fHoleY+iC) = 1857.0 + atof(cData[2])*17.57789; 
#ifdef DEBUG_FILE_READ_SBR
          fprintf(stdout, "%f %f \n", *(fHoleX+iC), *(fHoleY+iC));
#endif

            iC=iC+1;
         }
      }
   }

   if( iStart == 0 ){ iStat = -1; }
   else if( iC == 0 ){ iStat = -1; }
   else { iStat = iC; }

   fclose(pFile);
   return  iStat;
}
