/*****************************************************
 *  Aperture Photometory:
 *
 *    Read Subarray of a FITS image
 *
 *   Input : cFilename : FITS filename
 *   Input : pRange    : XY range (xmin, xmax, ymin, ymax)
 *   Output: pArray    : Sub array image
 *   Output:           : Return status from cfitsio
 *
 *     Version 2.0 20030830 Masayuki Akiyama from MMT
 *****************************************************
 */

#include  "acquisition.h"

#undef DEBUG

int fits_read_subarray(const char *cFilename, const int *pRange, float *pArray)
{
   fitsfile  *pFits;
   long       iAxlen[2]={1024,1024};
   int        iStat;
   int        i,j;
   int        iNdatax, iNdatay;
   int        iNreadx, iNready;
   float      fNullval;
   int        iAnynull;
   int        iNfound;
   float      *pLine;
   int        iNpixel;

   iStat = 0;
   fNullval = 0;

   if(fits_open_file(&pFits, cFilename, READONLY, &iStat)>0){
          fits_report_error(stderr, iStat);
          exit(8);
   }
   if(fits_read_keys_lng(pFits, "NAXIS", 1, 2, iAxlen, &iNfound, &iStat)>0){
          fits_report_error(stderr, iStat);
          exit(8);
   }

   iNdatax = iAxlen[0];
   iNdatay = iAxlen[1];

   iNreadx = *(pRange+1) - *(pRange+0) + 1;
   iNready = *(pRange+3) - *(pRange+2) + 1;

   pLine = (float *)malloc(sizeof(float)*(iNreadx+1));

   for( i=0; i<iNready ; i++ ){
     iNpixel =  iNdatax * (*(pRange+2)+i-1) + *(pRange+0);  /* The first pixel is "1" */

#ifdef DEBUG
      fprintf(stderr, "The first pixel number, %d\n",iNpixel);
#endif

     if(iNpixel > 0){
         if(fits_read_img(pFits, TFLOAT, iNpixel, iNreadx, &fNullval,
             pLine, &iAnynull, &iStat)>0 ){
                 fits_report_error(stderr, iStat);
                 exit(8);
         }
         for( j=0; j<iNreadx; j++ ){
             *(pArray + i*iNreadx + j) = *(pLine+j);
         }
     }
     else{
         for( j=0; j<iNreadx; j++){
           *(pArray + i*iNreadx + j) = 0.0;
         }
     }
  }
  if(fits_close_file(pFits, &iStat)>0 ){
       fits_report_error(stderr, iStat);
       exit(8);
  }

  free(pLine);

  return iStat;
}
