/*********************************************************
 * MOIRCS Acquisition: 
 *
 *     mes_star
 *
 *   Input : fits image name : FITS file image name
 *   Input : hole_pos_list   : Hole position file (MDP file)
 *   Input : outputfile      : Output filename
 *
 *       Ver.1.0 : 20050921 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

#undef DEBUG

int main( argc, argv )
int     argc;
char    *argv[];
{
   int    i,j,k, m,n;
   int    dNx, dNy;
   int    axlen[2];
   int    *pRange0;
   float  *pix_val; 

   float  fXcenter, fYcenter;
   float  fXshift, fYshift;
   float  fRstar;
   float  fRstar_new;
   float  fSky_ave, fSky_sig;

   int    iNite, iNite_max;
   float  fXcenter_stat, fYcenter_stat;
   float  fR_stat; /* Centering Radius */
   float  fR_stat_new; /* Centering Radius */
        
   int    iNdatax_cut, iNdatay_cut;
   float  *pix_val_cut; 
   int    iSize_pix_val_cut;
   int    *mask_val_cut;
   int    iSize_mask_val_cut;
   int    iPix;
   int    iNhole;
   float  *pHoleX, *pHoleY;

   float  fXcenter_cut, fYcenter_cut;
   float  fXcenter_new, fYcenter_new;

   float  fThresh;

   int    iCheck;
   int    iInterac;

   FILE   *pFile0;  
   char   cLine[1000];


   if(argc != 5){
      printf(" usage : mes_star [image] [hole_pos_list(X Y)] [outputfile] [interac?] \n");  
      printf("  hole_pos_list : MDP file \n");
      exit(1);
   }
   fRstar = sqrt((DEF_STAR_MOIRCS*DEF_STAR_MOIRCS + MDP_ERR*MDP_ERR)/2.0);
     
/*
 * Read Image Data
 */
   fits_read_size( argv[1], axlen );
#ifdef DEBUG_MES_STAR
   printf("%s Image size = %d %d \n", argv[1], axlen[0], axlen[1]);
#endif

   pix_val = (float *)malloc(sizeof(float)*(axlen[0] * axlen[1]));
  
   pRange0 = (int *)malloc(sizeof(int)*4);

   *(pRange0+0) = 0;
   *(pRange0+1) = axlen[0]-1;
   *(pRange0+2) = 0;
   *(pRange0+3) = axlen[1]-1;

   if(fits_read_subarray( argv[1], pRange0, pix_val ) < 0){
       exit(0);
   }

/*
 * STEP 0: Read Hole List
 */
    iNhole = file_count_data(argv[2]);
    pHoleX = (float *)malloc(sizeof(float) * iNhole);
    pHoleY = (float *)malloc(sizeof(float) * iNhole);
//    iNhole = file_read_mdp(argv[2], pHoleX, pHoleY, ALIGN_STAR_TAG);
    iNhole = file_read_sbr(argv[2], pHoleX, pHoleY, ALIGN_STAR_TAG);
#ifdef DEBUG_MES_STAR
    for( i=0; i<iNhole; i++ ){
       fprintf( stderr, "%f %f \n", *(pHoleX+i), *(pHoleY+i));
    }
#endif

#ifdef DEBUG_MES_STAR
    fprintf( stderr, "Number of acquisition stars in the SBR file %s = %d \n", argv[2], iNhole);
#endif

/*
 * STEP 1: Pick 1st hole and 
 *       :    shift coordinates of the holes with the measured position of the 1st hole
 */

    if(cpgbeg(0, "/xw", 1, 1) != 1){
         exit(EXIT_FAILURE);
    }
    cpgpap(11.5, 0.80);
    cpgask(0); 
    cpgpage(); 
    cpgbbuf(); 

    dNx = axlen[0];
    dNy = axlen[1];
    ac_pickstar(pix_val, axlen, cLine, iNhole, pHoleX, pHoleY);
    free(pix_val);
#ifdef DEBUG_MES_STAR
    for( i=0; i<iNhole; i++ ){
       fprintf( stderr, "%f %f \n", *(pHoleX+i), *(pHoleY+i));
    }
#endif


    cpgebuf(); 

//  Open record file
    pFile0 = fopen(argv[3],"at");  

    iNdatax_cut = ceil(SIZE_SUBARRAY*fRstar)+1;
    iNdatay_cut = ceil(SIZE_SUBARRAY*fRstar)+1;
    pix_val_cut = (float *)calloc((iNdatax_cut*iNdatay_cut), sizeof(float));
    mask_val_cut = (int *)calloc((iNdatax_cut*iNdatay_cut), sizeof(int));
    iSize_pix_val_cut = iNdatax_cut*iNdatay_cut;
    iSize_mask_val_cut = iNdatax_cut*iNdatay_cut;

    for(i=0; i<iNhole; i++){

/*
 * STEP 2 : Extract Subarray around a HOLE
 */

       iCheck = 0;

       *(pRange0+0) = (int) (*(pHoleX+i) - SIZE_SUBARRAY/2.0 * fRstar);
       *(pRange0+1) = (int) (*(pHoleX+i) + SIZE_SUBARRAY/2.0 * fRstar);
       *(pRange0+2) = (int) (*(pHoleY+i) - SIZE_SUBARRAY/2.0 * fRstar); 
       *(pRange0+3) = (int) (*(pHoleY+i) + SIZE_SUBARRAY/2.0 * fRstar);

       if( *(pRange0+0) < 1 || *(pRange0+0) >= axlen[0] ){ iCheck=1; }
       if( *(pRange0+1) < 1 || *(pRange0+1) >= axlen[0] ){ iCheck=1; }
       if( *(pRange0+2) < 1 || *(pRange0+2) >= axlen[1] ){ iCheck=1; }
       if( *(pRange0+3) < 1 || *(pRange0+3) >= axlen[1] ){ iCheck=1; }

       if( iCheck == 1){
           fprintf(stderr," mes_star : The hole #%d is outside of the image\n", i+1);
           fprintf(pFile0, "    0.0     0.0 \n");
       }
       else{
           iNdatax_cut = *(pRange0+1) - *(pRange0+0) + 1;
           iNdatay_cut = *(pRange0+3) - *(pRange0+2) + 1;
          
           for(m=0; m<iSize_pix_val_cut; m++){
               *(mask_val_cut+m)=0;
               *(pix_val_cut+m)=0.0;
           }

                 
           if(fits_read_subarray( argv[1], pRange0, pix_val_cut ) < 0){
               exit(0);
           }

           fXcenter_cut = *(pHoleX+i) - ( *(pRange0+0) - 0.5 );
           fYcenter_cut = *(pHoleY+i) - ( *(pRange0+2) - 0.5 );

           cpgpage(); 
           cpgbbuf(); 

           sprintf( cLine, "HOLE %d ", i+1 );
        
/*
 * STEP 3 : Determine Centering Region etc.
 */
           iInterac = 1;
           if( strcmp(argv[4], "no")==0 ){
              iInterac = 0;
           }
           ac_corre_img0(pix_val_cut, mask_val_cut, iNdatax_cut, iNdatay_cut,
                     fXcenter_cut,fYcenter_cut, cLine, iInterac); 
 
           cpgebuf(); /* end batch of output */   


/*
 * Repeat Step 4 & 5
 */
           iNite_max = 10;
           fR_stat = STAT_AREA * fRstar;
           fXcenter_stat = fXcenter_cut;
           fYcenter_stat = fYcenter_cut;

           for( iNite=0; iNite<iNite_max; iNite++ ){
/*
 * STEP 4 : Determin Threshold
 */ 
              ac_thresh1(pix_val_cut, mask_val_cut, iNdatax_cut,iNdatay_cut,
                      fR_stat, fXcenter_stat, fYcenter_stat, BACKGROUND_THRESH_STAR, &fThresh);
   
/*
 * STEP 5: Determin Centroid of the star
 */
      
              ac_center2(pix_val_cut, mask_val_cut, iNdatax_cut, iNdatay_cut,
                    fR_stat, fXcenter_stat, fYcenter_stat, fThresh,
                    &fXcenter_new, &fYcenter_new); 

#ifdef DEBUG_MESSTAR
     printf(" mes_star : %d %f %f %f \n", iNite, fThresh, fXcenter_new, fYcenter_new);
              fprintf(stderr, "Original center = %f %f \n", fXcenter_cut, fYcenter_cut);
              fprintf(stderr, "Revised center = %f %f \n", fXcenter_stat, fYcenter_stat);
              fprintf(stderr, "New center = %f %f \n", fXcenter_new, fYcenter_new);
#endif

              if(fXcenter_new>0 && fYcenter_new>0 ){
                 fXcenter_stat = fXcenter_new;
                 fYcenter_stat = fYcenter_new;
              }
/*
 * STEP 6: Determin size of the star
 */ 
              ac_radius1(pix_val_cut, mask_val_cut, iNdatax_cut, iNdatay_cut,
                    fR_stat, fXcenter_stat, fYcenter_stat, fThresh,
                    &fRstar_new); 

              fR_stat = STAT_AREA * fRstar_new * 0.5;
           }

/*
 * STEP 6 : Display the result
 */

           cpgpage(); 
           cpgbbuf(); 
       
           if( fXcenter_new==0 && fYcenter_new==0 ){
              fprintf(stderr, " mes_star : centering failed ... \n ");
           }
           if( ac_holeview(pix_val_cut, mask_val_cut, iNdatax_cut,iNdatay_cut,
                   &fXcenter_new, &fYcenter_new, fRstar_new,
                   cLine,
                   &fSky_ave, &fSky_sig) > 0 ){
             exit(0);
           }
 
           cpgebuf(); /* end batch of output */   

/*
 * STEP 7 : Record Measured Data
 */
           if( fXcenter_new > 0 && fYcenter_new > 0 ){
              fprintf(pFile0, "%7.1f %7.1f \n",
              fXcenter_new +(*(pRange0+0)-0.5), fYcenter_new +(*(pRange0+2)-0.5));
           }
           else{
              fprintf(pFile0, "    0.0     0.0 \n");
           }

       }
    }

    fclose(pFile0);

/*
 * STEP 7 : End PGPLOT
 */
           cpgend(); 

/*
 * Free Used Memory
 */
        free(pix_val_cut);
        free(mask_val_cut);
        free(pHoleX);
        free(pHoleY);

  return EXIT_SUCCESS;
}
