/*********************************************************
 * MOIRCS Acquisition:
 *
 *   Input : None : None
 *   Output: None : None
 *
 *       Ver.0.0 : 20050123 : Masayuki Akiyama
 **********************************************************
 */

#include "acquisition.h"

int main( argc, argv )
int     argc;
char    *argv[];
{
        int    i;
        int    dNx, dNy;
        int    axlen[2];
        int    *pRange0;
        float  *pix_val; 

        float  fRcen; /* Centering Radius */
        float  fXshift, fYshift;
        float  fRstar;
        float  fSky_ave, fSky_sig;
        
        int    iNdatax_cut, iNdatay_cut;
        int    iNhole;
        int    iPix;
        float  *pHoleX, *pHoleY, *pHoleR;
        float  xcenter_cut, ycenter_cut;
        float  xcenter_new, ycenter_new;
        float  xcenter_old, ycenter_old;
        float  *pix_val_cut; 
        int    *mask_val_cut;

        float  fThresh;

        FILE *pFile0;  
        char   cLine[1000];

        int    iCheck;
        int    iInterac;


    if(argc != 5){
	    printf("usage : mes_starhole [image] [hole_pos_list(X Y)] [outputfile] [interac?] \n"); 
	    printf("  hole_pos_list : result from mes_hole \n");
	    exit(1);
    }
    fRstar = DEF_STAR_MOIRCS;

/*
 * Read Image Data
 */
   fits_read_size( argv[1], axlen );

   pix_val = (float *)malloc(sizeof(float)*(axlen[0] * axlen[1]));
  
   pRange0 = (int *)malloc(sizeof(int)*4);

   *(pRange0+0) = 0;
   *(pRange0+1) = axlen[0]-1;
   *(pRange0+2) = 0;
   *(pRange0+3) = axlen[1]-1;


   if(fits_read_subarray( argv[1], pRange0, pix_val ) < 0){
       exit(0);
   }

#ifdef DEBUG
   fprintf(stderr, " mes_starhole : Start PGPLOT\n");
#endif

    if(cpgbeg(0, "/xw", 1, 1) != 1){
         exit(EXIT_FAILURE);
    }
    cpgask(0); 
    cpgpage(); 
    cpgbbuf(); 

    cpgebuf(); 

#ifdef DEBUG
   fprintf(stderr, " mes_starhole : Read Hole List (results from mes_hole) \n");
#endif

    iNhole = file_count_data(argv[2]);
    pHoleX = (float *)malloc(sizeof(float) * iNhole);
    pHoleY = (float *)malloc(sizeof(float) * iNhole);
    pHoleR = (float *)malloc(sizeof(float) * iNhole);
    file_read_hole(argv[2], pHoleX, pHoleY, pHoleR);

// Open record file
    pFile0 = fopen(argv[3],"at");  

    for(i=0; i<iNhole; i++){
#ifdef DEBUG
   fprintf(stderr, " mes_starhole : Extract Sub-array around a hole. \n");
#endif

       *(pRange0+0) = (int) (*(pHoleX+i) - SIZE_SUBARRAY/2.0 * *(pHoleR+i));
       *(pRange0+1) = (int) (*(pHoleX+i) + SIZE_SUBARRAY/2.0 * *(pHoleR+i));
       *(pRange0+2) = (int) (*(pHoleY+i) - SIZE_SUBARRAY/2.0 * *(pHoleR+i)); 
       *(pRange0+3) = (int) (*(pHoleY+i) + SIZE_SUBARRAY/2.0 * *(pHoleR+i));

       iCheck = 0;
       if( *(pRange0+0) < 1 || *(pRange0+0) >= axlen[0] ){ iCheck=1; }
       if( *(pRange0+1) < 1 || *(pRange0+1) >= axlen[0] ){ iCheck=1; }
       if( *(pRange0+2) < 1 || *(pRange0+2) >= axlen[1] ){ iCheck=1; }
       if( *(pRange0+3) < 1 || *(pRange0+3) >= axlen[1] ){ iCheck=1; }

       if( iCheck == 1){
           fprintf(stderr,"mes_star : The hole is outside of the image\n");
            fprintf(pFile0, "  0.000   0.000   0.000   0.000 \n");
       }
       else{
          iNdatax_cut = *(pRange0+1) - *(pRange0+0) + 1;
          iNdatay_cut = *(pRange0+3) - *(pRange0+2) + 1;

          pix_val_cut = (float *)malloc(sizeof(float)*(iNdatax_cut*iNdatay_cut));
          mask_val_cut = (int *)malloc(sizeof(int)*(iNdatax_cut*iNdatay_cut));

          if(fits_read_subarray( argv[1], pRange0, pix_val_cut ) < 0){
              exit(0);
          }

          for(iPix=0; iPix<(iNdatax_cut*iNdatay_cut); iPix++){
             *(mask_val_cut+iPix) = 0;
          }

          xcenter_cut = *(pHoleX+i) - ( *(pRange0+0) - 0.5 );
          ycenter_cut = *(pHoleY+i) - ( *(pRange0+2) - 0.5 );

          cpgpage(); 
          cpgbbuf(); 

          sprintf( cLine, "HOLE %d ", i+1 );
        
#ifdef DEBUG
   fprintf(stderr, " mes_starhole : Remove Bad Pixel etc. manually. \n");
#endif
          iInterac = 1;
          if( strcmp(argv[4], "no")==0 ){
              iInterac = 0;
          }
              ac_corre_img2(pix_val_cut,mask_val_cut,iNdatax_cut,iNdatay_cut,
                  xcenter_cut,ycenter_cut, *(pHoleR+i), cLine, iInterac); 
 
          cpgebuf(); /* end batch of output */   

#ifdef DEBUG
   fprintf(stderr, " mes_starhole : Detemine threshold inside hole. \n");
#endif
          fRcen = 1.0 * *(pHoleR+i);
          ac_thresh2(pix_val_cut,mask_val_cut,iNdatax_cut,iNdatay_cut,
                fRcen, xcenter_cut, ycenter_cut, &fThresh);

#ifdef DEBUG
   fprintf(stderr, " mes_starhole : Detemine centroid of the star with fRcen. \n");
#endif
          ac_center2(pix_val_cut, mask_val_cut, iNdatax_cut, iNdatay_cut,
                 fRcen, xcenter_cut, ycenter_cut, fThresh,
                 &xcenter_new, &ycenter_new); 

          xcenter_old = xcenter_new;
          ycenter_old = ycenter_new;
          fRcen = 2.0 * fRstar;
          ac_center2(pix_val_cut, mask_val_cut, iNdatax_cut, iNdatay_cut,
                 fRcen, xcenter_old, ycenter_old, fThresh,
                 &xcenter_new, &ycenter_new); 

          xcenter_old = xcenter_new;
          ycenter_old = ycenter_new;
          fRcen = 1.0 * fRstar;
          ac_center2(pix_val_cut, mask_val_cut, iNdatax_cut, iNdatay_cut,
                 fRcen, xcenter_old, ycenter_old, fThresh,
                 &xcenter_new, &ycenter_new); 

#ifdef DEBUG
   fprintf(stderr, " mes_starhole : Display the result with sigma=%f average=%f. \n", fSky_sig, fSky_ave);
#endif
          cpgpage(); 
          cpgbbuf(); 
       
          if( ac_starview(pix_val_cut, mask_val_cut, iNdatax_cut,iNdatay_cut,
                     xcenter_cut, ycenter_cut, &xcenter_new, &ycenter_new, *(pHoleR+i),
                     fRstar, cLine,
                     &fSky_ave, &fSky_sig) > 0 ){
            exit(0);
          }

#ifdef DEBUG
   fprintf(stderr, " mes_starhole : Record measured data in ARCSEC unit. \n");
#endif
         if( xcenter_new > 0 && ycenter_new > 0 ){
            fprintf(pFile0, "%8.3f %8.3f %8.3f %8.3f \n",
                *(pHoleX+i), *(pHoleY+i), 
                 (xcenter_new +(*(pRange0+0)-0.5)), (ycenter_new +(*(pRange0+2)-0.5)));
         }
         else{
            fprintf(pFile0, "   0.000    0.000    0.000    0.000 \n");
         }
       }
    }
    fclose(pFile0);  

// End PGPLOT
    cpgend(); 

// Free used memory
    free(pRange0);
    free(pix_val);
    free(pix_val_cut);
    free(mask_val_cut);
    free(pHoleX);
    free(pHoleY);
    free(pHoleR);

    return EXIT_SUCCESS;
}
