# cutspec.py - cutout 2D spectrum
# 2007. 4. 26 YOSHIKAWA, Tomohiro
# $Log: not supported by cvs2svn $
# Revision 1.5  2007/09/03 08:34:33  tomohiro
# add listonly parameter to cutspec()
#
# Revision 1.4  2007/08/23 04:22:41  tomohiro
# modify cutspec to use NAME column of mdp file for output file name
# add mdpexport
# modify _cutspec1 to accept MEF file
#
from mcsmdp import slit
from astrocat import mdp, datalist
from iraftool import geomap, fileutil
import sys,os,copy
from pyraf import iraf
import pyfits, numpy

def mdptran(mdpfile, outmdp, infile, fit_interactive=True):
    slitlist = mdp.open(mdpfile).readlines()
    slist = slit.SlitList(INST)
    slist.loaddata(slitlist)
    image = fileutil.fileparse(infile)[0]
    if len( pyfits.open(iraf.osfn(image))) > 1:
        XPA.display(image+"[1]")
    else:
        XPA.display(image)
    imfits = pyfits.open(image)
    detid = imfits[0].header['DET-ID']
    if detid == 1:
        slist.xyshift(-2048, 0)
        slist.rotate(-90)
        slist.slit_swapxy()
    elif detid == 2:
        slist.xyshift(-2048, -1536)
        slist.rotate(-90)
        slist.slit_swapxy()
    
    while(True):
        XPA.clreg()
        _maskplot(slist)
        reflist = []
        inlist = []
        #print slist.list[0]
        while(True):
            sys.stdout.write("select shape then return or \'q\' to quit: ")
            sys.stdout.flush()
            str = sys.stdin.readline()
            if str[0] == 'q' or str[0] == 'Q': break
            sp = XPA.getinfo(coord='image')
            if len(sp) != 1: continue
            x_slit = sp[0].x
            y_slit = sp[0].y
            #print x_slit, y_slit
            sys.stdout.write("click the object position: ")
            sys.stdout.flush()
            x_obj, y_obj = XPA.readcur()
            #print x_obj, y_obj
            
            reflist.append({'REF_X': x_obj, 'REF_Y': y_obj})
            inlist.append({'X': x_slit, 'Y': y_slit})
        if len(reflist) < 1: return
        geodb = geomap.geomap(reflist, inlist, fitgeometry='rotate', interactive=fit_interactive)
        shift_x=-geodb.data['xshift']
        shift_y=-geodb.data['yshift']
        rotate = geodb.data['xrotation']
        XPA.clreg()
        _maskplot(slist, shift_x=shift_x, shift_y=shift_y, rotate=rotate)
        #print slist.list[0]
        sys.stdout.write("type q to write mdp database to file: ")
        sys.stdout.flush()
        str = sys.stdin.readline()
        if str[0] == 'q' or str[0] == 'Q': break
    slist.xyshift(shift_x, shift_y)
    slist.rotate(rotate)
    #print slist.list[0]
    if detid == 1:
        slist.writemdp(outmdp, xmin=1, xmax=1792)
    else:
        slist.writemdp(outmdp, xmin=256, xmax=2048)

def maskplot(mdpfile, image=None, raw=False):
    slitlist = mdp.open(mdpfile).readlines()
    slist = slit.SlitList(INST)
    slist.loaddata(slitlist)
    detid = False
    if image != None and image != 'INDEF':
        XPA.display(image)
        imfits = pyfits.open(image)
        detid = imfits[0].header['DET-ID']

    if raw and detid:
        if detid == 1:
            slist.xyshift(-2048, 0)
            slist.rotate(-90)
            slist.slit_swapxy()
        elif detid == 2:
            slist.xyshift(-2048, -2048)
            slist.rotate(-90)
            slist.slit_swapxy()
    XPA.showslit(slist)

def cutspec(infile, mdpfile, name='INDEF', ext=0,
            margin=0.95, xmin=1, xmax=iraf.INDEF, ymin=1, ymax=iraf.INDEF,
            listonly=False):
    """
    infile: input filename
    mdpfile: MDP database file name
    name: object name to cutout
    ext: extention number to cutout
    margin: cut out margin
    xmin, xmax, ymin, ymax: minimum and maximum pixel to cut out
    listonly: output only cut out list
    """
    filelist = fileutil.fileparse(infile, base=True)

    ## for single slit data
    if mdpfile == 'INDEF':
        outlist = []
        for fitsfile in filelist:
            filebase = os.path.splitext(os.path.basename(fitsfile))[0]
            outname = "%s_%s_%02d"%(filebase, name, ext)
            if not listonly:
                if(os.access(fileutil.osimg(outname), os.F_OK)):
                    os.unlink(fileutil.osimg(outname))
                    print "delete: "+outname
                _cutspec_single(fitsfile, outname, ext=ext,
                                xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax)
            outlist.append(outname)
        return outlist

    ## for MOS data
    mdpdata = mdp.open(mdpfile).readlines()
    
    outlist = []
    for fitsfile in filelist:
        fits_f = pyfits.open(fileutil.osimg(fitsfile))
        #print fitsfile,ext
        hdr = fits_f[ext].header
        if xmax == iraf.INDEF:
            xmax = hdr['NAXIS1']
        if ymax == iraf.INDEF:
            ymax = hdr['NAXIS2']
        filebase = os.path.splitext(os.path.basename(fitsfile))[0]
        if name == 'INDEF':
            for data in mdpdata:
                if(data['SLIT_TYPE'] == 'hole' or
                   data['X'] > xmax or
                   data['X'] < xmin or
                   data['Y'] > ymax or
                   data['Y'] < ymin
                   ):
                    continue
                outname = "%s_%s_%02d"%(filebase, data['NAME'],ext)
                if not listonly:
                    if(os.access(fileutil.osimg(outname), os.F_OK)):
                        os.unlink(fileutil.osimg(outname))
                        print "delete: "+outname
                    _cutspec1(fitsfile, data, outname,
                              ext=ext, margin=margin, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax)
                outlist.append(outname)
        else:
            data = mdpdata.getbykey('NAME', name)
            if data == None:
                raise MCSMDPError, 'no such object name: '+name
            outname = "%s_%s_%02d"%(filebase, data['NAME'],ext)
            if not listonly:
                if(os.access(fileutil.osimg(outname), os.F_OK)):
                    os.unlink(fileutil.osimg(outname))
                    print "delete: "+outname
                _cutspec1(fitsfile, data, outname,
                          ext=ext, margin=margin, xmin=xmin, xmax=xmax, ymin=ymin, ymax=ymax)
            outlist.append(outname)
    return outlist

def cutwl(infile, outfile, ymin=1, ymax=iraf.INDEF, margin=20, filter='INDEF',
          crval=iraf.INDEF):
    """
    cut spectrum along wavelength with WCS
    """
    if filter != 'INDEF':
        wmax, wmin = INST.disperser.getfilterparam([filter])
    inlist  = fileutil.fileparse(infile, base=True)
    outlist = fileutil.fileparse(outfile, base=True)
    for inimg, outimg in zip(inlist, outlist):
        img, hdr = pyfits.getdata(fileutil.osimg(inimg), header=True)
        if filter == 'INDEF':
            wmax, wmin = INST.disperser.getfilterparam([hdr['FILTER01']])
        print wmax, wmin
        if ymax == iraf.INDEF:
            ymax = hdr['NAXIS2']
        #print hdr['CRPIX1'], hdr['CRVAL1'], hdr['CDELT1']
        xmax = int(hdr['CRPIX1'] + (wmax - hdr['CRVAL1'])/hdr['CDELT1'] + margin)
        xmin = int(hdr['CRPIX1'] + (wmin - hdr['CRVAL1'])/hdr['CDELT1'] - margin)
        print xmax, xmin
        if xmax > hdr['NAXIS1']: xmax = hdr['NAXIS1']
        if xmin < 1: xmin = 1
        print '%s[%i:%i][%i:%i] -> %s'%(inimg, xmin, xmax, ymin, ymax, outimg)
        
        if crval != iraf.INDEF:
            xlen  = len(img[0])
            ylen  = len(img)
            ylen1 = ymax - ymin
            xlen1 = xmax - xmin
            xlen1 += int((hdr['CRVAL1'] - crval)/hdr['CDELT1'])
            img1 = numpy.zeros((ylen1, xlen1))
            img1[0:ylen][int((hdr['CRVAL1'] - crval)/hdr['CDELT1']):xlen] = img[0:ylen, 0:xlen]
            hdr['CRVAL1'] = crval
            pyfits.writeto(fileutil.osimg(outimg), img1, hdr)
        else:
            pyfits.writeto(fileutil.osimg(outimg), img[ymin-1:ymax, xmin-1:xmax], hdr)

def mdpexport(inlist, outlist):
    """
    export 2d-spectrum to GIF file
    """
    statres = iraf.imstatistics(inlist, fields='midpt,stddev', format=False, Stdout=1)
    for infile, outfile, line in zip(fileutil.fileparse(inlist),
                                     fileutil.fileparse(outlist),
                                     statres):
        midpt, stddev = line.split()
        midpt = float(midpt)
        stddev = float(stddev)
        z1 = midpt - stddev / 2.
        z2 = midpt + stddev / 2.
        print 'z1='+str(z1), 'z2='+str(z2)
        iraf.export(infile,  outfile, format='gif', outban='setcmap(zscale(i1,%f,%f),"heat")'%(z1,z2))

##############
# SUBROUTINE #
##############
def _maskplot(slist, shift_x=0, shift_y=0, magnify=1,
             rotate=False, invert_x=False, invert_y=False):
    dlist = copy.deepcopy(slist)
    dlist.xyshift(shift_x=shift_x, shift_y=shift_y)
    dlist.magnify(magnify,magnify)
    if rotate:
        dlist.rotate(rotate)
    XPA.showslit(dlist)
    return dlist

def _cutspec1(fitsfile, mdpdata, outname, ext=0,
              margin=0.95, xmin=1, xmax=iraf.INDEF, ymin=1, ymax=iraf.INDEF):
    fits_f = pyfits.open(fileutil.osimg(fitsfile))
    img = fits_f[ext].data
    hdr = fits_f[ext].header
    if xmax == iraf.INDEF:  xmax = hdr['NAXIS1']
    if ymax == iraf.INDEF:  ymax = hdr['NAXIS2']
    cutmin = int(mdpdata['Y'] - mdpdata['WIDTH_Y']*margin/2.0)
    cutmax = int(mdpdata['Y'] + mdpdata['WIDTH_Y']*margin/2.0)
    if cutmin < ymin:
        cutmin = ymin
        print "slit minimum is out of range, slit is truncated."
    if cutmax >= ymax:
        cutmax = ymax
        print "slit maximum is out of range, slit is truncated."
    print '%s[%i:%i][%i:%i] -> %s'%(mdpdata['NAME'], xmin, xmax, cutmin, cutmax, outname)
    hdr['OBJECT'] = mdpdata['NAME']
    del hdr['BPM']
    pyfits.writeto(fileutil.osimg(outname), img[cutmin-1:cutmax-1, xmin-1:xmax-1], hdr)

def maskoffsetbyds9(mdpfile, outmdp):
    slitlist = mdp.open(mdpfile).readlines()
    slist.loaddata(slitlist, rotate=rotate, invert_x=invert_x, invert_y=invert_y)
    sys.stdout.write("click the slit position: ")
    sys.stdout.flush()
    x_slit, y_slit = XPA.readcur()
    print x_slit, y_slit
    sys.stdout.write("click the object position: ")
    sys.stdout.flush()
    x_obj, y_obj = XPA.readcur()
    print x_obj, y_obj

    slist.xyshift(shift_x=x_obj-x_slit, shift_y=y_obj-y_slit)
    XPA.clreg()
    XPA.showslit(slist)
    slist.writemdp(outmdp)

def _cutspec_single(fitsfile, outname, ext=0,
                    xmin=1, xmax=iraf.INDEF, ymin=1, ymax=iraf.INDEF):
    fits_f = pyfits.open(fileutil.osimg(fitsfile))
    img = fits_f[ext].data
    hdr = fits_f[ext].header
    if xmax == iraf.INDEF:  xmax = hdr['NAXIS1']
    if ymax == iraf.INDEF:  ymax = hdr['NAXIS2']
    print '%s[%i:%i][%i:%i] -> %s'%(fitsfile, xmin, xmax, ymin, ymax, outname)
    del hdr['BPM']
    pyfits.writeto(fileutil.osimg(outname), img[ymin-1:ymax-1, xmin-1:xmax-1], hdr)
