"""
mcsmdp_pyraf.py - pyraf interface of Mask Data Pileline for MOIRCS
2007.4.3   YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.7  2007/09/03 08:38:01  tomohiro
modify mdpdisplay for multiframe
delete precr, combspec, scombine
add mdpproc, mdpcombine, shiftcombine, mkweight, mdparith, mdpfcalib, onedspec2specdata, bandpass, mdpextract, mdpplot, mdp2dplot, mdprotate, mdpshift, mklist, wikitable

Revision 1.6  2007/06/19 10:11:36  tomohiro
remove combspecall

"""
from pyraf import iraf
from mcsmdp import *
#from mcsmdp import ds9, error, instrument, object, shape, slit
from iraftool import *
import sys

## mktemplate
iraf.task(DOLLARmktemplate = "$"+iraf.osfn("mcsmdp$bin/mktemplate.sh"))

## mesoffset binaries
iraf.task(DOLLARmesstar = "$"+iraf.osfn("mcsmdp$mesoffset/mes_star"))
iraf.task(DOLLERmeshole = "$"+iraf.osfn("mcsmdp$mesoffset/mes_hole"))
iraf.task(DOLLERmesstarhole = "$"+iraf.osfn("mcsmdp$mesoffset/mes_starhole"))
iraf.task(DOLLERresviewer = "$"+iraf.osfn("mcsmdp$mesoffset/res_viewer"))

##########
# grobal #
##########
def mdpinit():
    global INST, SLIT, XPA
    # instrument configuration object
    INST = instrument.LoadFile(iraf.osfn("mcsmdp$etc/instrument.conf"))
    # slit list
    SLIT = slit.SlitList(INST)
    # xpa connection object
    XPA = ds9.open()
    XPA.inst = INST
parfile = iraf.osfn("mcsmdp$pyraf/mdpinit.par")
iraf.IrafTaskFactory(taskname="mdpinit", value=parfile, function=mdpinit)
mdpinit()

def mdpdisplay(file, frame=1, multiframe=False,
               scale='linear', smode='zscale', z1=None, z2=None,
               cmap='BB', invert=False):
    if z1 == iraf.INDEF: z1 = None
    if z2 == iraf.INDEF: z2 = None
    filelist = fileutil.fileparse(file, ext=True)
    nfile = len(filelist)
    for fl,ext in filelist:
        fl = fileutil.osimg(fl)
        XPA.display("%s[%d]"%(fl,ext), frame, scale, smode, z1, z2, cmap, invert)
        if nfile > 1:
            if multiframe:
                frame += 1
            else:
                print "next? (Y/n)",
                ans = sys.stdin.readline()
                if ans[0] == 'n': break
            nfile -= 1
                
parfile = iraf.osfn("mcsmdp$pyraf/mdpdisplay.par")
iraf.IrafTaskFactory(taskname="mdpdisplay",value=parfile, function=mdpdisplay)

def mdptvmark(file, frame=None,
              coord='image', coord_unit='default',
              mark='circle', size=None, size_unit='default', color='yellow',
              label=False, number=False):
    if frame == 'INDEF': frame = None
    if size  == 'INDEF': size  = None
    XPA.tvmark(file, frame, coord, coord_unit, mark, size, size_unit, color, label, number)
parfile = iraf.osfn("mcsmdp$pyraf/mdptvmark.par")
iraf.IrafTaskFactory(taskname="mdptvmark",value=parfile, function=mdptvmark)

################
# mask design
################
execfile(iraf.osfn("mcsmdp$pyraf/maskdesign.py"))

parfile = iraf.osfn("mcsmdp$pyraf/mdpload.par")
iraf.IrafTaskFactory(taskname="mdpload", value=parfile, function=mdpload)
parfile = iraf.osfn("mcsmdp$pyraf/mdpsave.par")
iraf.IrafTaskFactory(taskname="mdpsave", value=parfile, function=mdpsave)
parfile = iraf.osfn("mcsmdp$pyraf/mdpsetwcs.par")
iraf.IrafTaskFactory(taskname="mdpsetwcs", value=parfile, function=mdpsetwcs)

parfile = iraf.osfn("mcsmdp$pyraf/slitshow.par")
iraf.IrafTaskFactory(taskname="slitshow", value=parfile, function=slitshow)
parfile = iraf.osfn("mcsmdp$pyraf/specshow.par")
iraf.IrafTaskFactory(taskname="specshow", value=parfile, function=specshow)
parfile = iraf.osfn("mcsmdp$pyraf/slithide.par")
iraf.IrafTaskFactory(taskname="slithide", value=parfile, function=slithide)
parfile = iraf.osfn("mcsmdp$pyraf/spechide.par")
iraf.IrafTaskFactory(taskname="spechide", value=parfile, function=spechide)
parfile = iraf.osfn("mcsmdp$pyraf/slitadd.par")
iraf.IrafTaskFactory(taskname="slitadd", value=parfile, function=slitadd)
parfile = iraf.osfn("mcsmdp$pyraf/holeadd.par")
iraf.IrafTaskFactory(taskname="holeadd", value=parfile, function=slitadd)
parfile = iraf.osfn("mcsmdp$pyraf/slitinfo.par")
iraf.IrafTaskFactory(taskname="slitinfo", value=parfile, function=slitinfo)
parfile = iraf.osfn("mcsmdp$pyraf/slitrm.par")
iraf.IrafTaskFactory(taskname="slitrm", value=parfile, function=slitrm)
parfile = iraf.osfn("mcsmdp$pyraf/slitmod.par")
iraf.IrafTaskFactory(taskname="slitmod", value=parfile, function=slitmod)
parfile = iraf.osfn("mcsmdp$pyraf/slitmove.par")
iraf.IrafTaskFactory(taskname="slitmove", value=parfile, function=slitmove)

parfile = iraf.osfn("mcsmdp$pyraf/fov.par")
iraf.IrafTaskFactory(taskname="fov", value=parfile, function=fov)
parfile = iraf.osfn("mcsmdp$pyraf/fovhide.par")
iraf.IrafTaskFactory(taskname="fovhide", value=parfile, function=fovhide)
parfile = iraf.osfn("mcsmdp$pyraf/fovupdate.par")
iraf.IrafTaskFactory(taskname="fovupdate", value=parfile, function=fovupdate)
parfile = iraf.osfn("mcsmdp$pyraf/disperser.par")
iraf.IrafTaskFactory(taskname="disperser", value=parfile, function=disperser)

parfile = iraf.osfn("mcsmdp$pyraf/redraw.par")
iraf.IrafTaskFactory(taskname="redraw", value=parfile, function=redraw)

parfile = iraf.osfn("mcsmdp$pyraf/objadd.par")
iraf.IrafTaskFactory(taskname="objadd", value=parfile, function=objadd)
parfile = iraf.osfn("mcsmdp$pyraf/objplot.par")
iraf.IrafTaskFactory(taskname="objplot", value=parfile, function=objplot)
parfile = iraf.osfn("mcsmdp$pyraf/objlist.par")
iraf.IrafTaskFactory(taskname="objlist", value=parfile, function=objlist)
parfile = iraf.osfn("mcsmdp$pyraf/objfind.par")
iraf.IrafTaskFactory(taskname="objfind", value=parfile, function=objfind)
parfile = iraf.osfn("mcsmdp$pyraf/objinfo.par")
iraf.IrafTaskFactory(taskname="objinfo", value=parfile, function=objinfo)

################
# BPM
################
execfile(iraf.osfn("mcsmdp$pyraf/bpm.py"))
parfile = iraf.osfn("mcsmdp$pyraf/mkbpm.par")
iraf.IrafTaskFactory(taskname="mkbpm", value=parfile, function=mkbpm)

################
# flat data
################
execfile(iraf.osfn("mcsmdp$pyraf/flat.py"))
parfile = iraf.osfn("mcsmdp$pyraf/mkdflat.par")
iraf.IrafTaskFactory(taskname="mkdflat", value=parfile, function=mkdflat)

################
# pre-reduction
################
execfile(iraf.osfn("mcsmdp$pyraf/pre.py"))
parfile = iraf.osfn("mcsmdp$pyraf/mdpproc.par")
iraf.IrafTaskFactory(taskname="mdpproc", value=parfile, function=mdpproc)
parfile = iraf.osfn("mcsmdp$pyraf/crrejection.par")
iraf.IrafTaskFactory(taskname="crrejection", value=parfile, function=crrej)

###################
# cutout spectrum
###################
execfile(iraf.osfn("mcsmdp$pyraf/cutspec.py"))

parfile = iraf.osfn("mcsmdp$pyraf/mdptran.par")
iraf.IrafTaskFactory(taskname="mdptran", value=parfile, function=mdptran)

parfile = iraf.osfn("mcsmdp$pyraf/maskplot.par")
iraf.IrafTaskFactory(taskname="maskplot", value=parfile, function=maskplot)

parfile = iraf.osfn("mcsmdp$pyraf/cutspec.par")
iraf.IrafTaskFactory(taskname="cutspec", value=parfile, function=cutspec)

parfile = iraf.osfn("mcsmdp$pyraf/cutwl.par")
iraf.IrafTaskFactory(taskname="cutwl", value=parfile, function=cutwl)

###################
# combine spectrum
###################
execfile(iraf.osfn("mcsmdp$pyraf/comb.py"))
parfile = iraf.osfn("mcsmdp$pyraf/mdpcombine.par")
iraf.IrafTaskFactory(taskname="mdpcombine", value=parfile, function=mdpcombine)
parfile = iraf.osfn("mcsmdp$pyraf/shiftcombine.par")
iraf.IrafTaskFactory(taskname="shiftcombine", value=parfile, function=shiftcombine)
parfile = iraf.osfn("mcsmdp$pyraf/mkweight.par")
iraf.IrafTaskFactory(taskname="mkweight", value=parfile, function=mkweight)

###################
# spectrum data
###################
execfile(iraf.osfn("mcsmdp$pyraf/mdparith.py"))
parfile = iraf.osfn("mcsmdp$pyraf/mdparith.par")
iraf.IrafTaskFactory(taskname="mdparith", value=parfile, function=mdparith)

execfile(iraf.osfn("mcsmdp$pyraf/standard.py"))
parfile = iraf.osfn("mcsmdp$pyraf/mdpfcalib.par")
iraf.IrafTaskFactory(taskname="mdpfcalib", value=parfile, function=mdpfcalib)
parfile = iraf.osfn("mcsmdp$pyraf/onedspec2specdata.par")
iraf.IrafTaskFactory(taskname="onedspec2specdata", value=parfile, function=onedspec2specdata)
parfile = iraf.osfn("mcsmdp$pyraf/bandpass.par")
iraf.IrafTaskFactory(taskname="bandpass", value=parfile, function=bandpass)
parfile = iraf.osfn("mcsmdp$pyraf/rescurve.par")
iraf.IrafTaskFactory(taskname="rescurve", value=parfile, function=rescurve)

execfile(iraf.osfn("mcsmdp$pyraf/mdpextract.py"))
parfile = iraf.osfn("mcsmdp$pyraf/mdpextract.par")
iraf.IrafTaskFactory(taskname="mdpextract", value=parfile, function=mdpextract)
parfile = iraf.osfn("mcsmdp$pyraf/mdpplot.par")
iraf.IrafTaskFactory(taskname="mdpplot", value=parfile, function=mdpplot)
parfile = iraf.osfn("mcsmdp$pyraf/mdp2dplot.par")
iraf.IrafTaskFactory(taskname="mdp2dplot", value=parfile, function=mdp2dplot)

###################
# others
###################
execfile(iraf.osfn("mcsmdp$pyraf/mdprotate.py"))
parfile = iraf.osfn("mcsmdp$pyraf/mdprotate.par")
iraf.IrafTaskFactory(taskname="mdprotate", value=parfile, function=mdprotate)
parfile = iraf.osfn("mcsmdp$pyraf/mdpshift.par")
iraf.IrafTaskFactory(taskname="mdpshift", value=parfile, function=mdpshift)

execfile(iraf.osfn("mcsmdp$pyraf/mklist.py"))
parfile = iraf.osfn("mcsmdp$pyraf/mklist.par")
iraf.IrafTaskFactory(taskname="mklist", value=parfile, function=mklist)
parfile = iraf.osfn("mcsmdp$pyraf/wikitable.par")
iraf.IrafTaskFactory(taskname="wikitable", value=parfile, function=wikitable)

execfile(iraf.osfn("mcsmdp$pyraf/cosmology.py"))
parfile = iraf.osfn("mcsmdp$pyraf/cosmology.par")
iraf.IrafTaskFactory(taskname="cosmology", value=parfile, function=cosmology)
