# pre.py - pre-reduction tasks
# YOSHIKAWA, Tomohiro
#$Log: not supported by cvs2svn $
#Revision 1.5  2007/09/13 02:25:28  tomohiro
#add do_cosmicray parameter to mdpproc()
#modify sanity check
#modify inter mediate file cleaning
#delete obsolete functions
#
#Revision 1.4  2007/09/03 08:39:28  tomohiro
#modify precr() to mdpproc()
#
#Revision 1.3  2007/06/19 10:12:56  tomohiro
#add sanity check
#

import os
from pyraf import iraf
import pyfits
from iraftool import fileutil,geomap
iraf.imred(_doprint=False)
iraf.crutil(_doprint=False)

def mdpproc(aframe, bframe,
            do_cosmicray=True,
            bpmask="../BPM/nlbpm1_FF86.fits",
            do_flat=True,
            flat="nrDomeFlatEveHK500_1.fits", calflat_pos=iraf.INDEF,
            do_distcor=True,
            gdb="dir_mcsred$DATABASE/mcsdistcrr1_mar07.dbs", gmp="mcsdistcrr1_mar07.gmp",
            gc_interpolant='nearest', gc_boundary='nearest', gc_constant=0,
            do_clean=True
            ):
    ## sanity check
    if((not os.access(iraf.osfn(gdb), os.F_OK)) and do_distcor):
        raise error.MCSMDPError, gdb+": no such file"
    if((not os.access(fileutil.osimg(flat), os.F_OK)) and do_flat):
        raise error.MCSMDPError, flat+": no such file"
    
    flist = zip(fileutil.fileparse(aframe, check=True, base=True),
                fileutil.fileparse(bframe, check=True, base=True))
    
    if calflat_pos > 99:
        raise error.MCSMDPError, "calflat_pos should be less than 100"
    
    for ff in flist:
        rmlist =[]
        af = ff[0]
        bf = ff[1]
        
        ## flat fielding
        if do_flat == True:
            aname=fileutil.basename(af)[0]
            bname=fileutil.basename(bf)[0]
            if calflat_pos == iraf.INDEF:
                flaname = "fl"+aname
                flbname = "fl"+bname
            else:
                flaname = "fl%02d%s"%(calflat_pos,aname)
                flbname = "fl%02d%s"%(calflat_pos,bname)
            
            if(os.access(fileutil.osimg(flaname), os.F_OK)): os.unlink(fileutil.osimg(flaname))
            if(os.access(fileutil.osimg(flbname), os.F_OK)): os.unlink(fileutil.osimg(flbname))
            print "flat fielding: "+flaname
            iraf.imarith(aname, '/', flat, flaname)
            print "flat fielding: "+flbname
            iraf.imarith(bname, '/', flat, flbname)
            
        else:
            flaname = af
            flbname = bf
        
        ## cosmic-ray rejection
        if do_cosmicray:
            craframe = "cr"+flaname
            crbframe = "cr"+flbname
            crrej(flaname, flbname, craframe, crbframe, bpmask)
            if do_flat:
                rmlist.append(flaname)
                rmlist.append(flbname)
        else:
            craframe = flaname
            crbframe = flbname
        
        ## AB subtruction
        abname = 'ab'+craframe
        if(os.access(fileutil.osimg(abname), os.F_OK)): os.unlink(fileutil.osimg(abname))
        print "a - b: "+abname
        iraf.imarith(craframe, '-', crbframe, abname)
        
        ## distortion correction
        if do_distcor:
            gcname = "gc"+abname
            if(os.access(fileutil.osimg(gcname), os.F_OK)): os.unlink(fileutil.osimg(gcname))
            geodb = geomap.GeoMapDB(iraf.osfn(gdb), gmp)
            print "distortion correction: "+gcname
            geomap.geotran(abname, gcname, geodb, interpolant=gc_interpolant, boundary=gc_boundary, constant=gc_constant)
            if do_flat:
                rmlist.append(abname)
            gcaframe = "gc"+craframe
            gcbframe = "gc"+crbframe
            if(os.access(fileutil.osimg(gcaframe), os.F_OK)): os.unlink(fileutil.osimg(gcaframe))
            geomap.geotran(craframe, gcaframe, geodb, interpolant=gc_interpolant, boundary=gc_boundary, constant=gc_constant)
            if(os.access(fileutil.osimg(gcbframe), os.F_OK)): os.unlink(fileutil.osimg(gcbframe))
            geomap.geotran(crbframe, gcbframe, geodb, interpolant=gc_interpolant, boundary=gc_boundary, constant=gc_constant)
            if do_cosmicray:
                rmlist.append(craframe)
                rmlist.append(crbframe)
            rmlist.append(abname)
        else:
            gcname = abname
            gcaframe = craframe
            gcbframe = crbframe

        bga = pyfits.open(fileutil.osimg(gcaframe))
        pyfits.append(fileutil.osimg(gcname), bga[0].data, bga[0].header)
        bgb = pyfits.open(fileutil.osimg(gcbframe))
        pyfits.append(fileutil.osimg(gcname), bgb[0].data, bgb[0].header)
        rmlist.append(gcaframe)
        rmlist.append(gcbframe)
        
        #clean
        if do_clean:
            for rfile in rmlist:
                if(os.access(fileutil.osimg(rfile), os.F_OK)): os.unlink(fileutil.osimg(rfile))

def crrej(inimage1, inimage2, outimage1, outimage2, bpm,
          navg=15, nrej=100, nbkg=5, nsig=10, var0=0, var1=0, var2=0,
          lcrsig=100, hcrsig=10, crgrow=0.0,
          bpmdir="BPM"
          ):
    """cosmic rays and badpixel rejection
    origin by M. Akiyama
    """
    if(not os.access(bpmdir, os.F_OK)): os.mkdir(bpmdir)

    for in1, in2, out1, out2 in zip(fileutil.fileparse(inimage1),
                                    fileutil.fileparse(inimage2),
                                    fileutil.fileparse(outimage1),
                                    fileutil.fileparse(outimage2),
                                    ):
        _crrej1(in1, in2, out1, bpm,
                navg, nrej, nbkg, nsig, var0, var1, var2,
                lcrsig, hcrsig, crgrow, bpmdir)
        if inimage2 is not None:
            _crrej1(in2, in1, out2, bpm,
                    navg, nrej, nbkg, nsig, var0, var1, var2,
                    lcrsig, hcrsig, crgrow, bpmdir)

def _crrej1(inimage1, inimage2, outimage, bpm,
            navg=15, nrej=100, nbkg=5, nsig=10, var0=0, var1=0, var2=0,
            lcrsig=100, hcrsig=10, crgrow=0.0,
            bpmdir="BPM"
            ):
    if(os.access(fileutil.osimg(outimage), os.F_OK)): os.unlink(fileutil.osimg(outimage))

    qframe   = "CRrej_qframe.fits"
    qmask    = "CRrej_qmask.pl"
    crmask   = "CRrej_crmask.pl"
    #qcrmask  = "CRrej_qcrmask.pl"
    qbpm     = bpmdir+'/'+fileutil.basename(inimage1)[0]+'.pl'
    if(os.access(fileutil.osimg(qframe), os.F_OK)): os.unlink(fileutil.osimg(qframe))
    if(os.access(fileutil.osimg(qmask), os.F_OK)): os.unlink(fileutil.osimg(qmask))
    if(os.access(fileutil.osimg(crmask), os.F_OK)): os.unlink(fileutil.osimg(crmask))
    #if(os.access(fileutil.osimg(qcrmask), os.F_OK)): os.unlink(fileutil.osimg(qcrmask))
    if(os.access(fileutil.osimg(qbpm), os.F_OK)): os.unlink(fileutil.osimg(qbpm))

    if inimage2 is not None:
        iraf.imarith( inimage1, "/", inimage2, qframe)
    else:
        qframe = inimage1
    iraf.imcopy(bpm, qmask)
    iraf.craverage.crmask = qmask
    iraf.craverage.average = ""
    iraf.craverage.sigma = ""
    iraf.craverage.nrej = nrej
    iraf.craverage.nbkg = nbkg
    iraf.craverage.nsig = nsig
    iraf.craverage.var0 = var0
    iraf.craverage.var1 = var1
    iraf.craverage.var2 = var2
    iraf.craverage.crval = 1
    iraf.craverage.lcrsig = lcrsig
    iraf.craverage.hcrsig = hcrsig
    iraf.craverage.crgrow = crgrow
    iraf.craverage(qframe, "" )
    
    iraf.imcopy(bpm, crmask)
    iraf.craverage.crmask = crmask
    iraf.craverage.average = ""
    iraf.craverage.sigma = ""
    iraf.craverage.nrej = nrej
    iraf.craverage.nbkg = nbkg
    iraf.craverage.nsig = nsig
    iraf.craverage.var0 = var0
    iraf.craverage.var1 = var1
    iraf.craverage.var2 = var2
    iraf.craverage.crval = 1
    iraf.craverage.lcrsig = lcrsig
    iraf.craverage.hcrsig = hcrsig
    iraf.craverage.crgrow = crgrow
    iraf.craverage(inimage1, "" )

    print 'iraf.imarith(%s, \'max\', %s, %s)'%(qmask, crmask, qbpm)
    iraf.imarith(qmask, 'max', crmask, qbpm)
    #iraf.imarith(qcrmask, 'max', bpm, qbpm)
    
    # for bug of iraf 2.12.2a
    fxframe = "CRrej_fxframe.fits"
    if(os.access(fileutil.osimg(fxframe), os.F_OK)): os.unlink(fileutil.osimg(fxframe))
    _fixpix(inimage1, fxframe, qbpm)
    
    _quadcorr(fxframe, outimage)

    #if(os.access(fileutil.osimg(qframe), os.F_OK)): os.unlink(fileutil.osimg(qframe))
    #if(os.access(fileutil.osimg(qmask), os.F_OK)): os.unlink(fileutil.osimg(qmask))
    #if(os.access(fileutil.osimg(crmask), os.F_OK)): os.unlink(fileutil.osimg(crmask))
    #if(os.access(fileutil.osimg(fxframe), os.F_OK)): os.unlink(fileutil.osimg(fxframe))
    #if(os.access(fileutil.osimg(qcrmask), os.F_OK)): os.unlink(fileutil.osimg(qcrmask))

def _fixpix(inimage, outimage, bpm):
    trframe = "fixpix_trframe.fits"
    trmask  = "fixpix_trmask.pl"
    if(os.access(fileutil.osimg(trframe), os.F_OK)): os.unlink(fileutil.osimg(trframe))
    if(os.access(fileutil.osimg(trmask), os.F_OK)): os.unlink(fileutil.osimg(trmask))
    iraf.imtranspose(inimage, trframe)
    iraf.imtranspose(bpm, trmask)
    iraf.fixpix.linterp = 1
    iraf.fixpix.cinterp = iraf.INDEF
    iraf.fixpix(trframe, trmask)
    iraf.imtranspose(trframe, outimage)
    iraf.hedit(outimage, 'BPM', bpm, add=True, verify=False, show=True, update=True)
    if(os.access(fileutil.osimg(trframe), os.F_OK)): os.unlink(fileutil.osimg(trframe))
    if(os.access(fileutil.osimg(trmask), os.F_OK)): os.unlink(fileutil.osimg(trmask))

def _quadcorr(inimage, outimage):
    """modify quadrant border along column
    inimage:  input image file name
    outimage: output image file name
    """
    infits = pyfits.open(fileutil.osimg(inimage))
    data = infits[0].data
    lindex = 0
    for line in data:
        if lindex < 1023:
            line[1023] = (line[1022]+line[1024])/2.
        elif lindex > 1024:
            line[1024] = (line[1023]+line[1025])/2.
        lindex +=1
    data[1023][1023] = (2*data[1022][1023] +   data[1025][1023])/3.
    data[1023][1024] = (  data[1022][1024] + 2*data[1025][1024])/3.
    data[1024][1023] = (2*data[1022][1023] +   data[1025][1023])/3.
    data[1024][1024] = (  data[1022][1024] + 2*data[1025][1024])/3.
    
    infits.writeto(fileutil.osimg(outimage))
