# fixlen.py
# fixed-length catalog utility libraries
# $Log: not supported by cvs2svn $
# Revision 1.2  2008/06/04 09:35:48  tomohiro
# add nstr parameter to readline
#
# Revision 1.1  2008/02/01 23:57:54  tomohiro
# fixed-length catalog utility libraries
#
#

import __builtin__,os
from types import *
import catalog
import datalist

class FixLen(catalog.Catalog):
    """FixLen catalog class
    """

    def __init__(self, file=None, column=[], colen=[], sp='_'):
        """function __init__
        initialise FixLen object
        file: FixLen catalog file name
        column: array of column name
        colen: array of column length
        sp: spaces is replaced to this string
        """
        # check and set column
        if len(column) < 1 or len(colen) < 1:
            raise TypeError, "non-zero length column is needed."
        self.column = column
        self.colen = colen
        self.sp = sp
        
        # check and set file name
        self.name = file
        if not os.path.exists(self.name):
            raise IOError, "no such data file: "+self.name

    def readline(self, nv=None, nstr=None):
        if nstr != None:
            nstr = ['', ' ', '  '] + nstr
        else:
            nstr = ['', ' ', '  ']
        for line in __builtin__.open(self.name):
            if line[0] == '\n' or line[0] == '#':
                continue
            outdata = []
            idx = 0
            for col, clen in zip(self.column,self.colen):
                data = line[idx:idx+clen]
                #print data+'|',
                data = data.strip()
                #print data+'|',
                data = catalog.str2data(data, nstr=nstr, nv=nv)
                #print str(data)+'|',
                if type(data) == StringType and self.sp != None:
                    data = data.replace(' ', self.sp)
                outdata.append((col,data))
                idx += clen
            #print ''
            yield dict(outdata)

def open(file=None, column=[], colen=[], sp='_'):
    """open fixed-length catalog file and returns FixLen object
    file: fixed-length catalog file name
    """
    return FixLen(file, column, colen, sp)
