# mdp.py
# MDP file utility libraries
# 2006.6.30 YOSHIKAWA, Tomohiro
# $Log: not supported by cvs2svn $
# Revision 1.4  2008/05/21 00:44:32  tomohiro
# add parameter nstr to readline()
#
# Revision 1.3  2008/02/01 23:42:56  tomohiro
# add nv, nstr parameters to readline()
#
# Revision 1.2  2007/07/17 02:32:38  tomohiro
# revise readlines to readline
#

import __builtin__
import catalog
import datalist
import string
from mcsmdp.error import *

class MDP(catalog.Catalog):
    """MDP file class
    """
    def __init__(self, file=None):
        """constructor
        file: MDP file name
        """
        self.name = file
        if self.name == None: return
        self.column = ['X','Y','WIDTH_X','WIDTH_Y','ANGLE','PRIORITY','SLIT_TYPE','NAME']
    def readline(self, nv=None, nstr=['None']):
        outlist = []
        index = 0
        self._file = __builtin__.open(self.name)
        ### skip header
        while(True):
            line = self._file.readline()
            if len(line) <= 0: break
            if line[0] == '#' or line[0] == '\n': continue
            line = line.strip()
            data = line.split()
            index = index+1
            try:
                data = line.split(',')[0]
                name = string.join(line.split(',')[1:], ',')
                #name = name.rstrip('\n')
                name = name.strip()
                data = data.split()
                cx = float(data.pop(0))
                cy = float(data.pop(0))
                wx = float(data.pop(0))
                wy = float(data.pop(0))
                pa = float(data.pop(0))
                pri = float(data.pop(0))
                type = data.pop(0)
            except (IndexError,ValueError):
                raise MCSMDPError, 'Bad file format in file: '+ self.name +' line: '+str(index)
            if type == 'B':
                slit_type = 'slit'
            elif type == 'C':
                slit_type = 'hole'
            else:
                raise MCSMDPError, 'Undefined slit type: '+type+'\n'
            yield {
                'X': cx,
                'Y': cy,
                'WIDTH_X': wx,
                'WIDTH_Y': wy,
                'ANGLE': pa,
                'PRIORITY': pri,
                'SLIT_TYPE': slit_type,
                'NAME': name}

    def readlines0(self):
        outlist = []
        index = 0
        for line in __builtin__.open(self.name):
            if line[0] == '#' or line[0] == '\n': continue
            line = line.strip()
            data = line.split()
            index = index+1
            try:
                data = line.split(',')[0]
                name = string.join(line.split(',')[1:], ',')
                #name = name.rstrip('\n')
                name = name.strip()
                data = data.split()
                cx = float(data.pop(0))
                cy = float(data.pop(0))
                wx = float(data.pop(0))
                wy = float(data.pop(0))
                pa = float(data.pop(0))
                pri = float(data.pop(0))
                type = data.pop(0)
            except (IndexError,ValueError):
                raise MCSMDPError, 'Bad file format in file: '+ self.name +' line: '+str(index)
            if type == 'B':
                slit_type = 'slit'
            elif type == 'C':
                slit_type = 'hole'
            else:
                raise MCSMDPError, 'Undefined slit type: '+type+'\n'
            outlist.append({
                'X': cx,
                'Y': cy,
                'WIDTH_X': wx,
                'WIDTH_Y': wy,
                'ANGLE': pa,
                'PRIORITY': pri,
                'SLIT_TYPE': slit_type,
                'NAME': name})
        return datalist.DataList(outlist)
    def writeto(self, datalist, name=None):
        if name == None: name = self.name
        f_out = __builtin__.open(name, 'w')
        for data in datalist:
            if data['SLIT_TYPE'] == 'slit':
                type = 'B'
            else:
                type = 'C'
            f_out.write("%f %f %f %f %f %i %s, %s\n"%(
                data['X'],
                data['Y'],
                data['WIDTH_X'],
                data['WIDTH_Y'],
                data['ANGLE'],
                data['PRIORITY'],
                type,
                data['NAME']
                ))
        f_out.close()
    
    def set2fits(self, file):
        pass
            
def open(file=None):
    return MDP(file)
def writedata(list, file):
    mfile = MDP(file)
    mfile.writeto(list, file)
