# spitzer.py
# Spitzer catalog utility libraries
# $Log: not supported by cvs2svn $
# Revision 1.2  2007/07/17 02:33:22  tomohiro
# modify bug in readline to break at EOF
#

import __builtin__
import catalog
import datalist

class Spitzer(catalog.Catalog):
    """Spitzer catalog class
    """
    ## Operations
    def __init__(self, file=None):
        """function __init__
        initialize Spitzer object
        file: Spitzer catalog file name
        """
        self.name = file
        if self.name == None: return
        ## set header
        header = []
        comment = ''
        for line in __builtin__.open(self.name):
            if line[0] != '\\':
                break
            line = line[1:]
            line = line.strip()
            eqind =  line.find('=')
            if eqind > 0:
                key = line[:eqind]
                value = line[eqind+1:]
                key = key.strip()
                value = value.strip(' \'')
                header.append((key, value))
            elif line[:7] == 'Comment':
                comment = comment + line[7:].strip() + '\n'
            else:
                header.append(('TITLE', line.strip()))
        if len(comment):
            header.append(('Comment', comment))
        self.header = dict(header)
        ## set column
        for line in __builtin__.open(self.name):
            if line[0] != '|':
                continue
            col_tmp = line.strip('| \n').split('|')
            self.column = []
            for col in col_tmp:
                self.column.append(col.strip())
            break
        
    def readline(self, nv=None, nstr=['None']):
        output_list = []
        for line in  __builtin__.open(self.name):
            if len(line) <= 0: break
            if line[0] == '\\' or line[0] == '|':
                continue
            tmp_outline = []
            line = line.strip()
            for col,data in zip(self.column, line.split()):
                data = catalog.str2data(data, nv=nv, nstr=nstr)
                tmp_outline.append((col,data))
            yield dict(tmp_outline)
#            output_list.append(dict(tmp_outline))
#        return datalist.DataList(output_list, self.header)

def open(file=None):
    """open spitzer catalog file and returns gaia object
    file: gaia catalog file name
    """
    return Spitzer(file)

