"""
bandmerge.py - merge catalog
YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
"""
import os,math
from pyraf import iraf
from astrocat import datalist
import wcsmatch

def catmerge_wcs(cat_input, cat_merge, tolerance, imagefile, matchname,
                 convwcs=False, xref=0, yref=0,
                 inraname='RA', indecname='Dec', mgraname='RA', mgdecname='Dec',
                 matchkeys=None, nulldata='N/A'):
    cat_input_xy = wcsmatch.setxy(cat_input, imagefile, raname=inraname, decname=indecname,
                                  xname='REF_X', yname='REF_Y', convwcs=convwcs)
    cat_merge_xy = wcsmatch.setxy(cat_merge, imagefile, raname=mgraname, decname=mgdecname,
                                  xname='REF_X', yname='REF_Y', convwcs=convwcs)
    return catmerge(cat_input_xy, cat_merge_xy, tolerance, matchname,
                    xref=xref, yref=yref,
                    inxname='REF_X', inyname='REF_Y', mgxname='REF_X', mgyname='REF_Y',
                    matchkeys=matchkeys, nulldata='N/A')

def catmerge(cat_input, cat_merge, tolerance, matchname,
             xref=0, yref=0,
             inxname='X', inyname='Y', mgxname='X', mgyname='Y',
             matchkeys=None, nulldata='N/A'):
    """
    merge catalog files by xyxymatch
    cat_input: input catalog data
    cat_merge: catalog data to merge
    inxname, inyname: x,y keyword name for infile
    mgxname, mgyname: x,y keyword name for mergefile
    tolerance: match tolerance for xyxymatch
    """
    ### define temporally files
    tmp_match = iraf.osfn(iraf.mktemp("tmp$matchmerge_match"))
    tmp_input = iraf.osfn(iraf.mktemp("tmp$matchmerge_input"))
    tmp_reference = iraf.osfn(iraf.mktemp("tmp$matchmerge_reference"))
    ### set matchkeys
    if matchkeys == None:
        matchkeys = []
        for key in cd_ref.column:
            if key != mgxname and key != mgyname:
                matchkeys.append(key)
    ### Sanitary check
    for mkey in matchkeys:
        for ikey in cat_input.keys():
            if mkey == ikey:
                raise ValueError, "duplicate key entry: "+mkey
    ### catalog match
    f_input = open(tmp_input, 'w')
    for data in cat_input:
        f_input.write(str(data[inxname])+"\t"+str(data[inyname])+"\n")
    f_input.close()
    f_ref = open(tmp_reference, 'w')
    for data in cat_merge:
        f_ref.write(str(data[mgxname])+"\t"+str(data[mgyname])+"\n")
    f_ref.close()
    
    iraf.xyxymatch(tmp_input, tmp_reference, tmp_match, tolerance,
                   xref=xref, yref=yref,
                   xcolumn=1, ycolumn=2, xrcolumn=1, yrcolumn=2, xformat="%13.3f", yformat="%13.3f",
                   )
    
    ### make matchlist
    print "making matchlist..."
    matchlist = []
    for line in open(tmp_match):
        if(line[0] == '#' or line[0] == '\n'): continue
        line = line.strip()
        refnum,innum = line.split()[4:6]
        refnum = int(refnum)
        innum = int(innum)
        #print refnum, innum
        matchlist.append({"REFNUM":refnum, "INNUM":innum})
    if len(matchlist) == 0:
        print "matching failed"
        return
    matchlist = datalist.DataList(matchlist)

    ### catalog merge
    mergelist = []
    for data, index in zip(cat_input, range(1, len(cat_input)+1)):
        mergedata = []
        for key in data.keys():
            mergedata.append((key, data[key]))
        matchdata = matchlist.getbykey('INNUM', index)
        if matchdata == None:
            #print str(data[inidname])+": no match"
            for mkey in matchkeys:
                mergedata.append((mkey, nulldata))
            mergedata.append((matchname+"_X", 'inf'))
            mergedata.append((matchname+"_Y", 'inf'))
            mergedata.append((matchname+"_DIST", 'inf'))
        else:
            #print str(data[inidname])+": matched with "+str(matchdata['REFNUM'])
            refdata = cat_merge[matchdata['REFNUM']-1]
            for mkey in matchkeys:
                mergedata.append((mkey, refdata[mkey]))
            dist = math.sqrt((data[inxname] - refdata[mgxname])**2 + (data[inyname] - refdata[mgyname])**2)
            mergedata.append((matchname+"_DIST", dist))
            mergedata.append((matchname+"_X", refdata[mgxname]))
            mergedata.append((matchname+"_Y", refdata[mgyname]))
        mergelist.append(dict(mergedata))

    for rfile in [tmp_input, tmp_reference, tmp_match]:
        if os.path.exists(rfile): os.unlink(rfile)
    
    return datalist.DataList(mergelist)

