"""
staralt.py - calculate StarAlt
YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.1  2007/09/24 09:31:27  tomohiro
calculate StarAlt


"""
import math
from RO.Astro.Sph import *
from RO.Astro.Tm import *
from RO import MathUtil
from astrocat import datalist
from astrotool import radec

class StarAlt:
    """class StarAlt
    calculate Az/Alt of the stars
    """
    def __init__(self, lat, long):
        """constructor
        initialize StarAlt
        lat: latitude of observer (degree)
        long: longitude of observer (degree)
        """
        self.lat = lat
        self.long = long
    
    def azalt(self, ra, dec, unixtime=None, ut1utc=0):
        """calculate Az/Alt of the star
        ra:  RA of the star (degree)
        dec: DEC of the star (degree)
        unixtime: UNIX timestamp (None=current time)
        ut1utc:   UT1 minus UTC

        return: (Az, Alt)
        """
        lst_deg = self.lst(unixtime, ut1utc)
        ha_deg = lst_deg - ra
        (az, alt), flug = AzAltFromHADec.azAltFromHADec((ha_deg, dec), self.lat)
        alt = abs(alt)  # bug of RO?
        return (az, alt)
        
    def lst(self, unixtime=None, ut1utc=0):
        """get local sideral time
        unixtime: UNIX timestamp (None=current time)
        ut1utc:   UT1 minus UTC

        return: LST in degree
        """
        utc = UTCFromPySec.utcFromPySec(unixtime)
        ut1 = utc + ut1utc
        return LASTFromUT1.lastFromUT1(ut1, self.long)

def standardstar(ra, dec, starlist,
                 unixtime=None, az=20, airmass=0.2,
                 lat=19.83, long=-155.48):
    """find standard star for the list
    """
    # staralt object
    SA = StarAlt(lat, long)
    
    # get Az/El for object
    az_obj, alt_obj = SA.azalt(ra, dec, unixtime)
    am_obj = 1./MathUtil.cosd(90-alt_obj)
    az_min = az_obj - az
    az_max = az_obj + az
    am_min = am_obj - airmass
    am_max = am_obj + airmass
    #print az_obj,am_obj

    outlist = []
    for stardata in starlist:
        ra_star = radec.hrstr2deg(stardata['RA'])
        dec_star = radec.degstr2deg(stardata['DEC'])
        az_star, alt_star = SA.azalt(ra_star, dec_star, unixtime)
        am_star = 1./MathUtil.cosd(90-alt_star)
        #print az_star, alt_star
        #print stardata['HIP_ID'], ra_star, dec_star, az_star, am_star
        if am_star > am_min and am_star < am_max:
            if az_min < 0:
                if az_star < (az_min + 360) and az_star > az_max: continue
            elif az_max > 360:
                if az_star < az_min and az_star > (az_max - 360): continue
            elif az_star < az_min or az_star > az_max: continue
            outdata = []
            for k, v in stardata.iteritems():
                outdata.append((k,v))
            outdata.append(('Az',  az_star))
            outdata.append(('airmass', am_star))
            outlist.append(dict(outdata))

    return outlist,  az_obj,am_obj
    #return datalist.DataList(outlist)

