# cui.py
# MCSMDP CUI interface
# 2006.07.14 YOSHIKAWA, Tomohiro
import string, os, sys
import cmd
import commands
import pylab

import packages

#import MCSMDP

def str2data(data):
    try:
        data = int(data)
    except ValueError:
        try:
            data = float(data)
        except ValueError:
            if data == 'None':
                data = None
            elif data == 'True':
                data = True
            elif data == 'False':
                data = False
            else:
                pass
    return data

class basepkg:
    tasklist=[]

class Cui(cmd.Cmd):
    """CUI for MCSMDP"""
    ### Attributes
    ## overrides
    prompt = 'base> '
    mode = 'base'

    pkg = basepkg
    MCSMDP = packages.MCSMDP
    
    ### Methods
    ## overrides
    def emptyline(self):
        """do nothing if emptyline
        """
        return False
    def completenames(self, text, *ignored):
        """complete commands and tasks
        """
        dotext = 'do_'+text
        builtinlist =  [a[3:] for a in self.get_names() if a.startswith(dotext)]
        tasklist = [a for a in self.get_tasknames() if a.startswith(text)]
        return builtinlist + tasklist
    def completedefault(self, text, line, bigidx, endidx):
        """complete the files in the current directory
        """
        text0 = line.split()[-1]
        head, tail = os.path.split(text0)
        if(os.path.isdir(head)):
            dir = head
        elif(len(head) == 0):
            dir = '.'
            head = '.'
        else:
            return []
        
        ret = []
        ltext = len(text)
        for file in os.listdir(dir):
            if file[:ltext] == text:
                if(os.path.isdir(head+'/'+file)):
                    file = file+'/'
                ret.append(file)
        return ret
    def precmd(self, line):
        cmd, arg, line = self.parseline(line)
        for task in self.pkg.tasklist:
            if task == cmd:
                line = 'task ' + self.mode + ' ' + line
                break
        return line
    def do_help(self, arg):
        if arg:
            for taskname in self.pkg.tasklist:
                if taskname == arg:
                    task = self.get_task(taskname)
                    doc=task.__doc__
                    if doc:
                        self.stdout.write("%s\n"%str(doc))
                    return
        else:
            names = self.get_tasknames()
            self.print_topics('tasks in package '+self.mode+':', names, 15,80)
        cmd.Cmd.do_help(self, arg)
        return
            
    ## internal functions
    def get_tasknames(self, mode=None):
        if mode == None:
            return self.pkg.tasklist
        else:
            pkg = getattr(packages, mode)
            return pkg.tasklist
    def get_task(self, name, mode=None):
        if mode == None:
            return getattr(self.pkg, name)
        else:
            pkg = getattr(packages, mode)
            return getattr(pkg, name)
        
    def parsearg(self, arg):
        arg = arg.strip()
        arglist = arg.split()
        index = 0
        args = []
        for a in arglist:
            if a.find('=') > 0: break
            args.append(str2data(a))
            index = index+1

        kwargs = []
        for a in arglist[index:]:
            if a.find('=') < 0:
                raise SyntaxError, 'non-keyword arg after keyword arg: '+a
            k, v = a.split('=')
            kwargs.append((k, str2data(v)))
        kwargs = dict(kwargs)
        return args, kwargs
            
    ## built-in commands
    # exit
    def do_exit(self, args):
        """exit shell"""
        return True
    def do_quit(self, args):
        """synonym of exit"""
        return self.do_exit(args)
    def do_EOF(self, args):
        """synonym of exit"""
        self.stdout.write("\n")
        return self.do_exit(args)
    # shell
    def do_shell(self, args):
        stat, output = commands.getstatusoutput("tcsh -c '" + args + "'")
        self.stdout.write(output + "\n")
        if(stat != 0):
            self.stdout.write("Error: "+str(stat)+"\n")
        return False
    # cd
    def do_cd(self, dir):
        """change the current directory
        syntax:
          cd [dir]
        args:
          dir: directory name to change
               (if empty, change to $HOME)
        """
        if(len(dir) == 0): dir = os.getenv('HOME')
        os.chdir(dir)
        return False
    def complete_cd(self, *args):
        return self.complete_ls(*args)
    # ls
    def do_ls(self, args):
        """display files in specified directory
        syntax:
          ls [files]
        args:
          files: file names to display
               (if empty, display current directory)
        """
        if(len(args) == 0):
            argv=['.']
        else:
            argv = args.split()
        for arg in argv:
            self.stdout.write("files in %s:\n" % (arg))
            count = 0
            for file in os.listdir(arg):
                if(os.path.isdir(arg+'/'+file)):
                    file = file+"/"
                if((count % 2) == 0):
                    if(len(file)<30):
                        self.stdout.write("%-30s" % (file))
                    else:
                        self.stdout.write("%s\n" % (file))
                        count += 1
                else:
                    if(len(file)<30):
                        self.stdout.write("%-30s\n" % (file))
                    else:
                        self.stdout.write("\n%s\n" % (file))
                        #count += 1
                count += 1
            if((count % 2) != 0): print '\n',
    
    def do_mode(self, mode):
        """enter to the specified mode
        """
        try:
            self.MCSMDP = self.pkg.MCSMDP
        except AttributeError:
            pass
        if mode == 'base':
            self.pkg = basepkg
        else:
            self.pkg = getattr(packages, mode)
        self.pkg.MCSMDP = self.MCSMDP
        self.mode = mode
        self.prompt = mode + '> '
    def complete_mode(self, text, line, bigidx, endidx):
        return  [a for a in packages.get_pkgnames() if a.startswith(text)]

    def do_task(self, arg):
        args, kwargs = self.parsearg(arg)
        mode, task, args = args[0], args[1], args[2:]
        func = self.get_task(task, mode=mode)
        func(*args, **kwargs)

if __name__ == "__main__":
    cui = Cui()
    while True:
        try:
            cui.cmdloop()
            break
        except:
            #type, value = sys.exc_info()[:2]
            #print type, ":",  value
            try:
                type, value, traceback = sys.exc_info()
                sys.excepthook(type, value, traceback)
            finally:
                del traceback
