"""
integrate - integration libraries
2007.06.19. YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.2  2008/03/19 10:34:50  tomohiro
add boundary, constant option to interpolate

Revision 1.1  2008/02/01 23:52:16  tomohiro
integration libraries

"""
import numpy

def interpolate(idata, ipt, rpt, boundary='nearest', constant=0, method='linear'):
    """
    interpolate idata(ipt) at rpt
     boundary: nearest|nan|constant
     constant: constant value for boundary=constant
    """
    ## for bug in searchsorted ??
    try:
        rpt = rpt.tolist()
    except AttributeError:
        pass
    ## set index
    ind = numpy.searchsorted(ipt, rpt) - 1
    zero = (ind == -1)
    mx   = (ind == len(ipt)-1)
    ind[zero] = 0
    ind[mx] = len(ipt)-2
    
    ## interpolate
    if method == 'linear':
        fract = (rpt - ipt[ind])/(ipt[ind+1]-ipt[ind])
        rdata = idata[ind] + fract*(idata[ind+1]-idata[ind])
    elif method == 'nearest':
        rdata = numpy.zeros(len(rpt))
        idx = (rpt-ipt[ind]) < (ipt[ind+1]-rpt)
        rdata[idx] = idata[ind][idx]
        rdata[idx==False] = idata[ind+1][idx==False]
    else:
        raise ValueError, 'Invalid method: '+str(method)

    ## boundary
    if boundary == 'nearest':
        rdata[zero] = idata[0]
        rdata[mx]   = idata[-1]
    elif boundary == 'nan':
        rdata[zero] = numpy.nan
        rdata[mx]   = numpy.nan
    elif boundary == 'constant':
        rdata[zero] = constant
        rdata[mx]   = constant
    else:
        raise ValueError, 'Invalid boundary: '+str(boundary)
    
    return rdata

def integrate(x, y):
    """
    integrate y along x with trapezoidal method
    """
    ind = numpy.arange(len(x))
    ind1 = ind-1
    h = x[ind[1:]] - x[ind1[1:]]
    return numpy.sum(h*(y[ind[1:]] + y[ind1[1:]])/2.)
