"""
mksfdb - make sfdb for emission lines
2007.07.17. YOSHIKAWA, Tomohiro
$Log: not supported by cvs2svn $
Revision 1.5  2008/08/30 04:23:50  tomohiro
modify the calclation of the initial centroid of Hbeta, and the inital FWHM of NII

Revision 1.4  2008/06/20 04:06:38  tomohiro
modify the unit of sub_fwhm from A to km/s for Hbeta

Revision 1.3  2008/03/19 10:35:18  tomohiro
add cont_kmax

Revision 1.2  2008/02/01 23:53:52  tomohiro
add function emission()
obsolete O3(), Halpha()

Revision 1.1  2007/09/03 08:47:39  tomohiro
make sfdb for emission lines

"""
import pyfits,pylab,numpy
from matplotlib.ticker import IndexLocator, FuncFormatter, NullFormatter, MultipleLocator
from iraftool import *
from spectool import specdata

c = 2.99792458e5 # speed of light (km/s)

def emission(slist, em_flux=None, em_center=None, em_fwhm=None, center_tolerance=100, fix_fwhm=False, cont_flux=0,
             cont_kmax=4.0e-22,
             em_type='emisson', sub_frac=1, sub_feature=True, sub_fwhm=None, name='emission', o3_4959=True,
             broad=False):
    wl = slist.field('WAVELENGTH')
    fl = slist.field('FLUX')
    sg = slist.field('NOISE')
    
    outdb = sfdb.SFDB()

    if em_flux == None:
        em_f = specdata.integrate(wl, fl) / 2.
    else:
        em_f = em_flux
    if em_center == None:
        em_c = specdata.integrate(wl, wl*fl)/specdata.integrate(wl,fl)
    else:
        em_c = float(em_center)
    if em_fwhm == None:
        em_fw = numpy.sqrt(specdata.integrate(wl, (wl-em_c)**2*fl)/specdata.integrate(wl,fl))*2*numpy.sqrt(2*numpy.log(2))/5.
    else:
        em_fw = em_fwhm
    if sub_frac == None:
        if em_type == 'OIII':
            sub_frac = 0.5
        elif em_type == 'Halpha':
            sub_frac = 0.1
    
    em_fv = em_fw/em_c*c
    print 'initial guess: flux=%.2e, center=%.2f, fwhm=%.2f A, (%.2f km/s)'%(em_f, em_c, em_fw, em_fv)
    
    if em_type == 'OIII':
        name = '[OIII] 5007'
    elif em_type == 'Halpha':
        name = 'Halpha 6563'
    
    ## main emission
    em = sfdb.Gaussian(comment=name)
    em.set_param('flux', (em_f, em_f/3., em_f*3, em_f*1e-2, 0.001, 0))
    if center_tolerance > 0:
        em.set_param('centroid', (em_c, em_c-center_tolerance, em_c+center_tolerance, 1.0, 0.001, 0))
    else:
        em.set_param('centroid', (em_c, em_c, em_c, 1.0, 0.001, -1))
    if fix_fwhm == True:
        em.set_param('fwhm', (em_fv, em_fv, em_fv, 0.01, 0.001, -1))
    else:
        em.set_param('fwhm', (em_fv, em_fv/3., em_fv*3, em_fv*1e-2, 0.001, 0))
    em.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
    outdb.add_component(em)

    if em_type == 'OIII':
        ## [OIII]4959
        if o3_4959:
            o32_f = em_f/3.
            o32_c = em_c * 4959./5007.
            o32_fv = em_fw/o32_c*c
            o32 = sfdb.Gaussian(comment='[OIII] 4959')
            o32.set_param('flux', (o32_f, o32_f/3., o32_f*3, 1./3., 0.001, 0))
            o32.set_param('centroid', (o32_c, o32_c-center_tolerance, o32_c+center_tolerance, 4959./5007., 0.001, 1))
            o32.set_param('fwhm', (o32_fv, o32_fv/3., o32_fv*3, 1, 0.001, 1))
            o32.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
            outdb.add_component(o32)
        
        if sub_feature:
            ## H beta
            hb_f = em_f*sub_frac
            hb_c = em_c * 4861./5007.
            if sub_fwhm == None:
                hb_fv = em_fw/hb_c*c
            else:
                hb_fv = sub_fwhm/hb_c*c
            hb = sfdb.Gaussian(comment='Hbeta 4861')
            hb.set_param('flux', (hb_f, hb_f/3., hb_f*3, hb_f*1e-2, 0.001, 0))
            hb.set_param('centroid', (hb_c, hb_c-center_tolerance, hb_c+center_tolerance, 4861./5007., 0.001, 1))
            if sub_fwhm == None:
                hb.set_param('fwhm', (hb_fv, hb_fv/3., hb_fv*3, hb_fv*1e-2, 0.001, 0))
            else:
                hb.set_param('fwhm', (hb_fv, hb_fv, hb_fv, 0.01, 0.001, -1))
            hb.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
            outdb.add_component(hb)
    elif em_type == 'Halpha':
        if sub_feature:
            ## [NII]
            n21_f = em_f*sub_frac
            n21_c = em_c * 6583./6563.
            if sub_fwhm == None:
                n21_fv = em_fw/n21_c*c
            else:
                n21_fv = sub_fwhm/n21_c*c
            n21 = sfdb.Gaussian(comment='[NII] 6583')
            n21.set_param('flux', (n21_f, n21_f/3., n21_f*3, n21_f*1e-2, 0.001, 0))
            n21.set_param('centroid', (n21_c, n21_c-center_tolerance, n21_c+center_tolerance, 6583./6563., 0.001, 1))
            if sub_fwhm == None:
                n21.set_param('fwhm', (n21_fv, n21_fv/3., n21_fv*3, n21_fv*1e-2, 0.001, 0))
            else:
                n21.set_param('fwhm', (n21_fv, n21_fv, n21_fv, 0.01, 0.001, -1))
            n21.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
            outdb.add_component(n21)
            
            n22_c = em_c * 6548./6563.
            n22_fv = em_fw/n22_c*c
            n22 = sfdb.Gaussian(comment='[NII] 6548')
            n22.set_param('flux', (n21_f/3., n21_f/9., n21_f, 0.3333, 0.001, 2))
            n22.set_param('centroid', (n22_c, n22_c-center_tolerance, n22_c+center_tolerance, 6548./6563., 0.001, 1))
            if sub_fwhm == None:
                n22.set_param('fwhm', (n22_fv, n22_fv/3., n22_fv*3, 1, 0.001, 2))
            else:
                n22.set_param('fwhm', (n21_fv, n21_fv, n21_fv, 0.01, 0.001, -1))
            n22.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
            outdb.add_component(n22)

    ## broad Ha emission
    if broad and em_type == 'Halpha':
        emb = sfdb.Gaussian(comment='Halpha 6563 broad')
        emb.set_param('flux', (em_f*5, em_f*5/3., em_f*5*3, em_f*1e-2, 0.001, 0))
        emb.set_param('centroid', (em_c, em_c, em_c, 1.0, 0.001, 1))
        emb.set_param('fwhm', (em_fv*10, em_fv*10/3., em_fv*30., em_fv*1e-2, 0.001, 0))
        emb.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
        outdb.add_component(emb)
    
    k    = 0
    kmin = -cont_kmax
    kmax =  cont_kmax
    fmin = kmin*(em_c-1000) + cont_flux
    fmax = kmax*(em_c-1000) + cont_flux
    cont = sfdb.Linear(comment='continuum')
    cont.set_param('flux', (cont_flux, fmin, fmax, fmax*1e-2, 0.001, 0))
    cont.set_param('slope', (0, kmin, kmax, kmax*1e-2, 0.001, 0))
    outdb.add_component(cont)

    return outdb

def O3(slist, o3_flux=None, o3_center=None, o3_fwhm=None, center_tolerance=100,
       cont_flux=0, hb=True, hb_frac=0.5):
    wl = slist.field('WAVELENGTH')
    fl = slist.field('FLUX')
    sg = slist.field('NOISE')
    
    outdb = sfdb.SFDB()

    if o3_flux == None:
        o3_f = specdata.integrate(wl, fl) / 2.
    else:
        o3_f = o3_flux
    if o3_center == None:
        o3_c = specdata.integrate(wl, wl*fl)/specdata.integrate(wl,fl)
    else:
        o3_c = o3_center
    if o3_fwhm == None:
        o3_fw = numpy.sqrt(specdata.integrate(wl, (wl-o3_c)**2*fl)/specdata.integrate(wl,fl))*2*numpy.sqrt(2*numpy.log(2))/5.
    else:
        o3_fw = o3_fwhm
    if hb_frac == None:
        hb_frac = 0.1
    o3_fv = o3_fw/o3_c*c
    print 'initial guess: flux=%.2e, center=%.2f, fwhm=%.2f A, (%.2f km/s)'%(o3_f, o3_c, o3_fw, o3_fv)

    ## [OIII]5007
    o31 = sfdb.Gaussian(comment='[OIII] 5007')
    o31.set_param('flux', (o3_f, o3_f/3., o3_f*3, o3_f*1e-2, 0.001, 0))
    if center_tolerance > 0:
        o31.set_param('centroid', (o3_c, o3_c-center_tolerance, o3_c+center_tolerance, 1.0, 0.001, 0))
    else:
        o31.set_param('centroid', (o3_c, o3_c, o3_c, 1.0, 0.001, -1))
    o31.set_param('fwhm', (o3_fv, o3_fv/3., o3_fv*3, o3_fv*1e-2, 0.001, 0))
    o31.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
    outdb.add_component(o31)

    ## [OIII]4959
    o32_f = o3_f/3.
    o32_c = o3_c * 4959./5007.
    o32_fv = o3_fw/o32_c*c
    o32 = sfdb.Gaussian(comment='[OIII] 4959')
    o32.set_param('flux', (o32_f, o32_f/3., o32_f*3, 1./3., 0.001, 0))
    o32.set_param('centroid', (o32_c, o32_c-center_tolerance, o32_c+center_tolerance, 4959./5007., 0.001, 1))
    o32.set_param('fwhm', (o32_fv, o32_fv/3., o32_fv*3, 1, 0.001, 1))
    o32.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
    outdb.add_component(o32)

    if hb:
        hb_f = o3_f*hb_frac
        hb_c = o3_c * 4861./6563.
        hb_fv = o3_fw/hb_c*c
        hb = sfdb.Gaussian(comment='Hb 4861')
        hb.set_param('flux', (hb_f, hb_f/3., hb_f*3, hb_f*1e-2, 0.001, 0))
        hb.set_param('centroid', (hb_c, hb_c-center_tolerance, hb_c+center_tolerance, 4861./5007., 0.001, 1))
        hb.set_param('fwhm', (hb_fv, hb_fv/3., hb_fv*3, hb_fv*1e-2, 0.001, 0))
        hb.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
        outdb.add_component(hb)
    
    k    = 0
    kmin = -4.0e-22
    kmax =  4.0e-22
    fmin = kmin*(o3_c-1000) + cont_flux
    fmax = kmax*(o3_c-1000) + cont_flux
    cont = sfdb.Linear(comment='continuum')
    cont.set_param('flux', (cont_flux, fmin, fmax, fmax*1e-2, 0.001, 0))
    cont.set_param('slope', (0, kmin, kmax, kmax*1e-2, 0.001, 0))
    outdb.add_component(cont)

    return outdb

def Halpha(slist, ha_flux=None, ha_center=None, ha_fwhm=None, center_tolerance=100,
           cont_flux=0, n2=True, n2_frac=0.1):
    wl = slist.field('WAVELENGTH')
    fl = slist.field('FLUX')
    sg = slist.field('NOISE')
    
    outdb = sfdb.SFDB()

    if ha_flux == None:
        ha_f = specdata.integrate(wl, fl) / 2.
    else:
        ha_f = ha_flux
    if ha_center == None:
        ha_c = specdata.integrate(wl, wl*fl)/specdata.integrate(wl,fl)
    else:
        ha_c = ha_center
    if ha_fwhm == None:
        ha_fw = numpy.sqrt(specdata.integrate(wl, (wl-ha_c)**2*fl)/specdata.integrate(wl,fl))*2*numpy.sqrt(2*numpy.log(2))/5.
    else:
        ha_fw = ha_fwhm
    if n2_frac == None:
        n2_frac = 0.1
    ha_fv = ha_fw/ha_c*c
    print 'initial guess: flux=%.2e, center=%.2f, fwhm=%.2f A, (%.2f km/s)'%(ha_f, ha_c, ha_fw, ha_fv)
    ha = sfdb.Gaussian(comment='Halpha 6563')
    ha.set_param('flux', (ha_f, ha_f/3., ha_f*3, ha_f*1e-2, 0.001, 0))
    if center_tolerance > 0:
        ha.set_param('centroid', (ha_c, ha_c-center_tolerance, ha_c+center_tolerance, 1.0, 0.001, 0))
    else:
        ha.set_param('centroid', (ha_c, ha_c, ha_c, 1.0, 0.001, -1))
    ha.set_param('fwhm', (ha_fv, ha_fv/3., ha_fv*3, ha_fv*1e-2, 0.001, 0))
    ha.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
    outdb.add_component(ha)

    if n2:
        n21_f = ha_f*n2_frac
        n21_c = ha_c * 6583./6563.
        n21_fv = ha_fw/n21_c*c
        n21 = sfdb.Gaussian(comment='[NII] 6583')
        n21.set_param('flux', (n21_f, n21_f/3., n21_f*3, n21_f*1e-2, 0.001, 0))
        n21.set_param('centroid', (n21_c, n21_c-center_tolerance, n21_c+center_tolerance, 6583./6563., 0.001, 1))
        n21.set_param('fwhm', (n21_fv, n21_fv/3., n21_fv*3, n21_fv*1e-2, 0.001, 0))
        n21.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
        outdb.add_component(n21)
        
        n22_c = ha_c * 6548./6563.
        n22_fv = ha_fw/n22_c*c
        n22 = sfdb.Gaussian(comment='[NII] 6548')
        n22.set_param('flux', (n21_f/3., n21_f/9., n21_f, 0.3333, 0.001, 2))
        n22.set_param('centroid', (n22_c, n22_c-center_tolerance, n22_c+center_tolerance, 6548./6563., 0.001, 1))
        n22.set_param('fwhm', (n22_fv, n22_fv/3., n22_fv*3, 1, 0.001, 2))
        n22.set_param('skew', (1.0, 1.0, 1.0, 0.01, 0.001, -1))
        outdb.add_component(n22)
    
    k    = 0
    kmin = -4.0e-22
    kmax =  4.0e-22
    fmin = kmin*(ha_c-1000) + cont_flux
    fmax = kmax*(ha_c-1000) + cont_flux
    cont = sfdb.Linear(comment='continuum')
    cont.set_param('flux', (cont_flux, fmin, fmax, fmax*1e-2, 0.001, 0))
    cont.set_param('slope', (0, kmin, kmax, kmax*1e-2, 0.001, 0))
    outdb.add_component(cont)

    return outdb
